# Generated by Django 4.2.13 on 2024-07-26 06:40

import datetime
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    replaces = [
        ("gestion", "0001_initial"),
        ("gestion", "0002_livresupplement"),
        ("gestion", "0003_livresupplement_commentaire_livre_a_lire"),
        ("gestion", "0004_alter_livre_a_lire_options"),
        ("gestion", "0005_bibliotheque_localisation"),
        ("gestion", "0006_remove_localisation_bib_courriers_memoire_and_more"),
    ]

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Caution",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "numero",
                    models.IntegerField(unique=True, verbose_name="numéro de caution"),
                ),
                ("nom", models.CharField(max_length=50, verbose_name="Nom de l'élève")),
                ("prenom", models.CharField(max_length=25, verbose_name="Prénom")),
                ("date", models.DateField(verbose_name="Date d'inscription à la coop")),
                (
                    "classe",
                    models.CharField(max_length=25, verbose_name="Classe initiale"),
                ),
                (
                    "rendue_le",
                    models.DateField(
                        blank=True,
                        default=datetime.date(1, 1, 1),
                        verbose_name="Rendue le",
                    ),
                ),
                (
                    "somme_payee",
                    models.FloatField(
                        blank=True, default=70, verbose_name="Somme payée"
                    ),
                ),
                (
                    "somme_rendue",
                    models.FloatField(
                        blank=True, default=0, verbose_name="Somme rendue"
                    ),
                ),
                (
                    "commentaire",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=140,
                        verbose_name="Commentaire",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Classes",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "code",
                    models.CharField(max_length=25, verbose_name="Code de la Classe"),
                ),
                (
                    "libelle",
                    models.CharField(
                        max_length=25, verbose_name="Libellé de la Classe"
                    ),
                ),
                (
                    "compte",
                    models.IntegerField(
                        blank=True, default=0, null=True, verbose_name="Compte De Noms"
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Classes",
            },
        ),
        migrations.CreateModel(
            name="Commandes",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                ("date", models.CharField(max_length=25, verbose_name="Date")),
                (
                    "commentaire",
                    models.CharField(max_length=25, verbose_name="Commentaire"),
                ),
            ],
            options={
                "verbose_name_plural": "Commandes",
            },
        ),
        migrations.CreateModel(
            name="Disciplines",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "libelle",
                    models.CharField(
                        max_length=25, verbose_name="Libellé de la Discipline"
                    ),
                ),
                (
                    "abrege",
                    models.CharField(
                        max_length=25, verbose_name="Abrégé de la Discipline"
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Disciplines",
            },
        ),
        migrations.CreateModel(
            name="Eleves",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "sexe",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Sexe"
                    ),
                ),
                (
                    "Eleve_No_Etab",
                    models.IntegerField(
                        blank=True, null=True, unique=True, verbose_name="N° d'Élève"
                    ),
                ),
                (
                    "Num_Eleve_Etab",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        unique=True,
                        verbose_name="N° d'Élève+Étab.",
                    ),
                ),
                (
                    "Nom",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Nom"
                    ),
                ),
                (
                    "Nom_de_famille",
                    models.CharField(max_length=50, verbose_name="Nom de famille"),
                ),
                (
                    "Nom_d_usage",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Nom d'usage"
                    ),
                ),
                ("Prenom", models.CharField(max_length=25, verbose_name="Prénom")),
                (
                    "Prenom_2",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Prénom 2"
                    ),
                ),
                (
                    "Prenom_3",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Prénom 3"
                    ),
                ),
                (
                    "NomPrenomEleve",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="Concaténation du Nom et du Prénom",
                    ),
                ),
                (
                    "sans_accent",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="Nom sans accents",
                    ),
                ),
                (
                    "Date_Naissance",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date de Naissance"
                    ),
                ),
                (
                    "Doublement",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Doublement"
                    ),
                ),
                (
                    "Id_National",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Id National"
                    ),
                ),
                (
                    "Date_Entree",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date d' Entrée"
                    ),
                ),
                (
                    "Date_Sortie",
                    models.DateField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Date de Sortie",
                    ),
                ),
                (
                    "Code_Regime",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Code Régime"
                    ),
                ),
                (
                    "Lib_Regime",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Libellé Régime",
                    ),
                ),
                (
                    "Code_MEF",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Code MEF"
                    ),
                ),
                (
                    "Lib_MEF",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Libellé MEF"
                    ),
                ),
                (
                    "Code_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Code de Structure",
                    ),
                ),
                (
                    "Type_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Type Structure",
                    ),
                ),
                (
                    "Lib_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Libellé Structure",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_1",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 1",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_1",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 1",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_2",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 2",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_2",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 2",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_3",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 3",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_3",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 3",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_4",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 4",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_4",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 4",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_5",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 5",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_5",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 5",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_6",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 6",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_7",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 7",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_8",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 8",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_9",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 9",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_10",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 10",
                    ),
                ),
                (
                    "Tel_Personnel",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Personnel",
                    ),
                ),
                (
                    "Tel_Professionnel",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Professionnel",
                    ),
                ),
                (
                    "Tel_Portable",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Portable",
                    ),
                ),
                (
                    "Email",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Email"
                    ),
                ),
                (
                    "Adresse",
                    models.TextField(blank=True, null=True, verbose_name="Adresse"),
                ),
                (
                    "Civilite_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité du Représentant Légal",
                    ),
                ),
                (
                    "Nom_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom du Représentant Légal",
                    ),
                ),
                (
                    "Nom_de_famille_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom de famille du Représentant Légal",
                    ),
                ),
                (
                    "Nom_d_usage_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom d'usage du Représentant Légal",
                    ),
                ),
                (
                    "Prenom_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Prénom Représentantant Légal",
                    ),
                ),
                (
                    "ConcaReprLeg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="ConcaReprLég",
                    ),
                ),
                (
                    "Tel_Personnel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Personnel Repr Lég",
                    ),
                ),
                (
                    "Lien_Parente_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Lien Parenté Repr Lég",
                    ),
                ),
                (
                    "Tel_Portable_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Portable Repr Lég",
                    ),
                ),
                (
                    "Tel_Professionnel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Professionnel Repr Lég",
                    ),
                ),
                (
                    "Email_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Email Repr Lég",
                    ),
                ),
                (
                    "Communication_Adresse_postale_et_courriel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Communication Adresse postale et courriel Repr Lég",
                    ),
                ),
                (
                    "Adresse_Repr_Leg",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="Adresse du représentant légal",
                    ),
                ),
                (
                    "Civilite_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_de_famille_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom de famille Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_d_usage_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom d'usage Autre Repr Lég",
                    ),
                ),
                (
                    "Prenom_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Prénom Autre Repr Lég",
                    ),
                ),
                (
                    "ConcaAutreReprLeg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="ConcaAutreReprLég",
                    ),
                ),
                (
                    "Tel_Personnel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Personnel Autre Repr Lég",
                    ),
                ),
                (
                    "Lien_Parente_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Lien Parenté Autre Repr Lég",
                    ),
                ),
                (
                    "Tel_Portable_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Portable Autre Repr Lég",
                    ),
                ),
                (
                    "Tel_Professionnel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Professionnel Autre Repr Lég",
                    ),
                ),
                (
                    "Email_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Email Autre Repr Lég",
                    ),
                ),
                (
                    "Communication_Adresse_postale_et_courriel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Communication Adresse postale et courriel Autre Repr Lég",
                    ),
                ),
                (
                    "Adresse_Autre_Repr_Leg",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="Adresse d'un autre représentant légal",
                    ),
                ),
                (
                    "date_inscription_coop",
                    models.DateField(
                        blank=True,
                        null=True,
                        verbose_name="Date d'inscription à la coop",
                    ),
                ),
                (
                    "no_caution",
                    models.IntegerField(default=15000, verbose_name="N° de la caution"),
                ),
                (
                    "a_verifier",
                    models.BooleanField(
                        default=True, verbose_name="À vérifier manuellement"
                    ),
                ),
                (
                    "autorisation_provisoire_inscription",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        verbose_name="Autorisation provisoire de s'inscrire",
                    ),
                ),
                (
                    "classes_passees",
                    models.CharField(
                        default="", max_length=50, verbose_name="Classes passées"
                    ),
                ),
                (
                    "classe",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.classes",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Élèves",
            },
        ),
        migrations.CreateModel(
            name="Eleves_de_siecle",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "sexe",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Sexe"
                    ),
                ),
                (
                    "Eleve_No_Etab",
                    models.IntegerField(
                        blank=True, null=True, unique=True, verbose_name="N° d'Élève"
                    ),
                ),
                (
                    "Num_Eleve_Etab",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        unique=True,
                        verbose_name="N° d'Élève+Étab.",
                    ),
                ),
                (
                    "Nom",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Nom"
                    ),
                ),
                (
                    "Nom_de_famille",
                    models.CharField(max_length=50, verbose_name="Nom de famille"),
                ),
                (
                    "Nom_d_usage",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Nom d'usage"
                    ),
                ),
                ("Prenom", models.CharField(max_length=25, verbose_name="Prénom")),
                (
                    "Prenom_2",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Prénom 2"
                    ),
                ),
                (
                    "Prenom_3",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Prénom 3"
                    ),
                ),
                (
                    "NomPrenomEleve",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="Concaténation du Nom et du Prénom",
                    ),
                ),
                (
                    "sans_accent",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="Nom sans accents",
                    ),
                ),
                (
                    "Date_Naissance",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date de Naissance"
                    ),
                ),
                (
                    "Doublement",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Doublement"
                    ),
                ),
                (
                    "Id_National",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Id National"
                    ),
                ),
                (
                    "Date_Entree",
                    models.DateField(
                        blank=True, null=True, verbose_name="Date d' Entrée"
                    ),
                ),
                (
                    "Date_Sortie",
                    models.DateField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Date de Sortie",
                    ),
                ),
                (
                    "Code_Regime",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Code Régime"
                    ),
                ),
                (
                    "Lib_Regime",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Libellé Régime",
                    ),
                ),
                (
                    "Code_MEF",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Code MEF"
                    ),
                ),
                (
                    "Lib_MEF",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Libellé MEF"
                    ),
                ),
                (
                    "Code_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Code de Structure",
                    ),
                ),
                (
                    "Type_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Type Structure",
                    ),
                ),
                (
                    "Lib_Structure",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Libellé Structure",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_1",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 1",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_1",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 1",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_2",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 2",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_2",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 2",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_3",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 3",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_3",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 3",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_4",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 4",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_4",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 4",
                    ),
                ),
                (
                    "Cle_Gestion_Mat_Enseignee_5",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Clé Matière Enseignée 5",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_5",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 5",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_6",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 6",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_7",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 7",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_8",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 8",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_9",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 9",
                    ),
                ),
                (
                    "Lib_Mat_Enseignee_10",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Matière Enseignée 10",
                    ),
                ),
                (
                    "Tel_Personnel",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Personnel",
                    ),
                ),
                (
                    "Tel_Professionnel",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Professionnel",
                    ),
                ),
                (
                    "Tel_Portable",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Téléphone Portable",
                    ),
                ),
                (
                    "Email",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Email"
                    ),
                ),
                (
                    "Adresse",
                    models.TextField(blank=True, null=True, verbose_name="Adresse"),
                ),
                (
                    "Civilite_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité du Représentant Légal",
                    ),
                ),
                (
                    "Nom_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom du Représentant Légal",
                    ),
                ),
                (
                    "Nom_de_famille_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom de famille du Représentant Légal",
                    ),
                ),
                (
                    "Nom_d_usage_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom d'usage du Représentant Légal",
                    ),
                ),
                (
                    "Prenom_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Prénom Représentantant Légal",
                    ),
                ),
                (
                    "ConcaReprLeg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="ConcaReprLég",
                    ),
                ),
                (
                    "Tel_Personnel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Personnel Repr Lég",
                    ),
                ),
                (
                    "Lien_Parente_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Lien Parenté Repr Lég",
                    ),
                ),
                (
                    "Tel_Portable_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Portable Repr Lég",
                    ),
                ),
                (
                    "Tel_Professionnel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Professionnel Repr Lég",
                    ),
                ),
                (
                    "Email_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Email Repr Lég",
                    ),
                ),
                (
                    "Communication_Adresse_postale_et_courriel_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Communication Adresse postale et courriel Repr Lég",
                    ),
                ),
                (
                    "Adresse_Repr_Leg",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="Adresse du représentant légal",
                    ),
                ),
                (
                    "Civilite_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_de_famille_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom de famille Autre Repr Lég",
                    ),
                ),
                (
                    "Nom_d_usage_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom d'usage Autre Repr Lég",
                    ),
                ),
                (
                    "Prenom_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Prénom Autre Repr Lég",
                    ),
                ),
                (
                    "ConcaAutreReprLeg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="ConcaAutreReprLég",
                    ),
                ),
                (
                    "Tel_Personnel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Personnel Autre Repr Lég",
                    ),
                ),
                (
                    "Lien_Parente_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Lien Parenté Autre Repr Lég",
                    ),
                ),
                (
                    "Tel_Portable_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Portable Autre Repr Lég",
                    ),
                ),
                (
                    "Tel_Professionnel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Tél Professionnel Autre Repr Lég",
                    ),
                ),
                (
                    "Email_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Email Autre Repr Lég",
                    ),
                ),
                (
                    "Communication_Adresse_postale_et_courriel_Autre_Repr_Leg",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Communication Adresse postale et courriel Autre Repr Lég",
                    ),
                ),
                (
                    "Adresse_Autre_Repr_Leg",
                    models.TextField(
                        blank=True,
                        null=True,
                        verbose_name="Adresse d'un autre représentant légal",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Élèves_de_siecle",
            },
        ),
        migrations.CreateModel(
            name="Filieres",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "nom",
                    models.CharField(max_length=25, verbose_name="Nom de la Filière"),
                ),
            ],
            options={
                "verbose_name_plural": "Filières",
            },
        ),
        migrations.CreateModel(
            name="Inventaire",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "Date_Achat_Materiel",
                    models.DateField(verbose_name="Date d'Achat du Materiel"),
                ),
                (
                    "NumeroDeSerie",
                    models.CharField(max_length=25, verbose_name="Numero de Série"),
                ),
                ("Perdu", models.BooleanField(default=False, verbose_name="Perdu")),
                (
                    "pilon",
                    models.DateField(
                        default=datetime.date(1, 1, 1), verbose_name="Mis au pilon, le"
                    ),
                ),
                (
                    "Etat_Materiel",
                    models.IntegerField(
                        choices=[
                            (1, "Neuf"),
                            (2, "Bon"),
                            (3, "Passable"),
                            (4, "Mauvais"),
                            (5, "Très Mauvais"),
                            (6, "Incomplet"),
                            (0, "Autre"),
                        ],
                        default=0,
                        verbose_name="État du matériel",
                    ),
                ),
                (
                    "a_imprimer",
                    models.BooleanField(default=True, verbose_name="à imprimer"),
                ),
            ],
            options={
                "verbose_name_plural": "Inventaire",
            },
        ),
        migrations.CreateModel(
            name="Jeton",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("duree", models.IntegerField(verbose_name="Durée en secondes")),
                (
                    "perime",
                    models.DateTimeField(
                        default=datetime.datetime(
                            2000, 12, 31, 23, 0, tzinfo=datetime.timezone.utc
                        ),
                        verbose_name="Périmé après",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Niveaux",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "libelle",
                    models.CharField(max_length=25, verbose_name="Libellé du Niveau"),
                ),
                (
                    "abrege",
                    models.CharField(max_length=25, verbose_name="Abrégé du Niveau"),
                ),
                (
                    "rang",
                    models.CharField(max_length=25, verbose_name="Rang du Niveau"),
                ),
            ],
            options={
                "verbose_name_plural": "Niveaux",
            },
        ),
        migrations.CreateModel(
            name="Personnels",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("civilite", models.CharField(max_length=25, verbose_name="Civilité")),
                (
                    "nom",
                    models.CharField(
                        max_length=25, verbose_name="Nom de l'Accompagnateur"
                    ),
                ),
                (
                    "prenom",
                    models.CharField(
                        max_length=25, verbose_name="Prénom de l'Accompagnateur"
                    ),
                ),
                (
                    "nomprenom",
                    models.CharField(
                        max_length=25,
                        verbose_name="Concaténation du Nom et du Prénom de l'Accompagnateur",
                    ),
                ),
                (
                    "tel",
                    models.CharField(
                        max_length=25, verbose_name="Téléphone du Domicile"
                    ),
                ),
                (
                    "mobile",
                    models.CharField(
                        max_length=25, verbose_name="Mobile de l'Accompagnateur"
                    ),
                ),
                (
                    "courriel_pro",
                    models.CharField(
                        max_length=25,
                        verbose_name="Courriel professionnel de l'accompagnateur",
                    ),
                ),
                (
                    "courriel_perso",
                    models.CharField(
                        max_length=25,
                        verbose_name="Courriel personnel de l'accompagnateur",
                    ),
                ),
                (
                    "discipline",
                    models.CharField(max_length=25, verbose_name="Discipline"),
                ),
            ],
            options={
                "verbose_name_plural": "Accompagnateurs",
            },
        ),
        migrations.CreateModel(
            name="Series",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "libelle_court",
                    models.CharField(
                        max_length=25, verbose_name="Libellé Court de la Série"
                    ),
                ),
                (
                    "libelle_long",
                    models.CharField(
                        max_length=25, verbose_name="Libellé Long de la Série"
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Séries",
            },
        ),
        migrations.CreateModel(
            name="TypeMateriel",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "libelle_long",
                    models.CharField(
                        max_length=25, verbose_name="Libellé Long du Type de Matériel"
                    ),
                ),
                (
                    "libelle_court",
                    models.CharField(
                        max_length=25, verbose_name="Libellé Court du Type de Matériel"
                    ),
                ),
                (
                    "Tarif_Perdu",
                    models.CharField(
                        max_length=25, verbose_name="Tarif pour une perte"
                    ),
                ),
                (
                    "Tarif_Degradation",
                    models.CharField(
                        max_length=25, verbose_name="Tarif pour une dégradation"
                    ),
                ),
                (
                    "Choix_Tarif_Degradation",
                    models.BooleanField(
                        verbose_name="Appliquer le tarif de dégradation"
                    ),
                ),
                (
                    "Appliquer_Demi_Tarif",
                    models.BooleanField(verbose_name="Appliquer un demi-tarif"),
                ),
            ],
            options={
                "verbose_name_plural": "Types de matériel",
            },
        ),
        migrations.CreateModel(
            name="Prets",
            fields=[
                (
                    "id",
                    models.IntegerField(
                        primary_key=True, serialize=False, verbose_name="index"
                    ),
                ),
                ("date_pret", models.DateField(verbose_name="Date de prêt")),
                (
                    "date_retour",
                    models.DateField(
                        blank=True,
                        default=datetime.date(1, 1, 1),
                        max_length=25,
                        verbose_name="Date de retour",
                    ),
                ),
                (
                    "Date_Declaration_Perte",
                    models.DateField(
                        blank=True,
                        default=datetime.date(1, 1, 1),
                        max_length=25,
                        verbose_name="Date de déclaration de la perte",
                    ),
                ),
                (
                    "Etat_Initial",
                    models.IntegerField(
                        choices=[
                            (1, "Neuf"),
                            (2, "Bon"),
                            (3, "Passable"),
                            (4, "Mauvais"),
                            (5, "Très Mauvais"),
                            (6, "Incomplet"),
                            (0, "Autre"),
                        ],
                        default=0,
                        null=True,
                        verbose_name="État Initial",
                    ),
                ),
                (
                    "Etat_Final",
                    models.IntegerField(
                        blank=True,
                        choices=[
                            (1, "Neuf"),
                            (2, "Bon"),
                            (3, "Passable"),
                            (4, "Mauvais"),
                            (5, "Très Mauvais"),
                            (6, "Incomplet"),
                            (0, "Autre"),
                        ],
                        default=0,
                        verbose_name="État Final",
                    ),
                ),
                ("Remise", models.FloatField(default=0, verbose_name="Remise")),
                (
                    "accompagnateur",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.personnels",
                    ),
                ),
                (
                    "eleve",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.eleves",
                    ),
                ),
                (
                    "inventaire",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.inventaire",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Prêts",
            },
        ),
        migrations.CreateModel(
            name="Parametres",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "NomAcademie",
                    models.CharField(max_length=25, verbose_name="Nom de l'Académie"),
                ),
                (
                    "NomEts",
                    models.CharField(
                        max_length=25, verbose_name="Nom de l'Établissement"
                    ),
                ),
                ("CodeRNE", models.CharField(max_length=25, verbose_name="Code RNE")),
                ("Commune", models.CharField(max_length=25, verbose_name="Commune")),
                (
                    "Titre_ChefEts",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Titre du Chef d'Établissement",
                    ),
                ),
                (
                    "Civilite_ChefEts",
                    models.CharField(
                        max_length=25, verbose_name="Civilité du Chef d'Établissement"
                    ),
                ),
                (
                    "NomChefEts",
                    models.CharField(
                        max_length=25, verbose_name="Nom du Chef d'Établissement"
                    ),
                ),
                (
                    "Civilite_AgentComptable",
                    models.CharField(
                        max_length=25, verbose_name="Civilité de l'Agent Comptable"
                    ),
                ),
                (
                    "NomAgentComptable",
                    models.CharField(
                        max_length=25, verbose_name="Nom de l'Agent Comptable"
                    ),
                ),
                (
                    "EtsSupportAgence",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Agence de support de l'établissement",
                    ),
                ),
                (
                    "IBAN",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="IBAN"
                    ),
                ),
                (
                    "BIC",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="BIC"
                    ),
                ),
                (
                    "Titre_Gestionnaire",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Titre  du Gestionnaire",
                    ),
                ),
                (
                    "Civilite_Gestionnaire",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité du Gestionnaire",
                    ),
                ),
                (
                    "NomGestionnaire",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom du Gestionnaire",
                    ),
                ),
                (
                    "CoordoneesIntendance",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Coordonnées de l'Intendance",
                    ),
                ),
                (
                    "Version_Application",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Version de l'Application",
                    ),
                ),
                (
                    "AnneeScolaire",
                    models.CharField(max_length=25, verbose_name="Année Scolaire"),
                ),
                (
                    "AnneeScolaireEnPreparation",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Année Scolaire En Préparation",
                    ),
                ),
                ("Adresse", models.CharField(max_length=25, verbose_name="Adresse")),
                (
                    "CodePostal",
                    models.CharField(max_length=25, verbose_name="Code Postal"),
                ),
                (
                    "Telephone",
                    models.CharField(max_length=25, verbose_name="Téléphone"),
                ),
                (
                    "Fax",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Fax"
                    ),
                ),
                (
                    "SiteWeb",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Site Web"
                    ),
                ),
                (
                    "Nom_Assistant_social",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Nom de l'Assistant social",
                    ),
                ),
                (
                    "Civilite_Assistant_social",
                    models.CharField(
                        blank=True,
                        max_length=25,
                        null=True,
                        verbose_name="Civilité de l'Assistant social",
                    ),
                ),
                (
                    "Logo",
                    models.ImageField(
                        blank=True, null=True, upload_to="./media", verbose_name="Logo"
                    ),
                ),
                (
                    "signature",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="signature"
                    ),
                ),
            ],
            options={
                "verbose_name": "Paramètres",
                "verbose_name_plural": "Paramètres",
                "unique_together": {("CodeRNE", "AnneeScolaire")},
            },
        ),
        migrations.CreateModel(
            name="Materiel",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                (
                    "titre",
                    models.CharField(max_length=50, verbose_name="Titre (ou nom)"),
                ),
                (
                    "editeur",
                    models.CharField(max_length=25, verbose_name="Editeur ou Marque"),
                ),
                (
                    "Prix",
                    models.CharField(
                        blank=True,
                        max_length=15,
                        null=True,
                        verbose_name="Prix du Materiel",
                    ),
                ),
                (
                    "Annee_Depot_Legal",
                    models.CharField(
                        blank=True,
                        max_length=5,
                        null=True,
                        verbose_name="Année du Dépôt Légal",
                    ),
                ),
                ("ISBN", models.CharField(max_length=25, verbose_name="ISBN")),
                (
                    "Collection",
                    models.CharField(
                        blank=True, max_length=25, null=True, verbose_name="Collection"
                    ),
                ),
                (
                    "Auteurs",
                    models.TextField(blank=True, null=True, verbose_name="Auteurs"),
                ),
                (
                    "Serie_loc",
                    models.CharField(
                        blank=True,
                        max_length=10,
                        null=True,
                        verbose_name="Série (locale)",
                    ),
                ),
                (
                    "Option",
                    models.CharField(
                        blank=True, max_length=10, null=True, verbose_name="Option"
                    ),
                ),
                (
                    "Pretes",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="nombre de manuels prêtés"
                    ),
                ),
                (
                    "Achetes",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        verbose_name="nombre de manuels total achetés",
                    ),
                ),
                (
                    "Perdus",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="nombre de manuels perdus"
                    ),
                ),
                (
                    "epaisseur_mm",
                    models.IntegerField(default=15, verbose_name="épaisseur en mm"),
                ),
                (
                    "largeur_cm",
                    models.IntegerField(default=20, verbose_name="largeur en cm"),
                ),
                (
                    "hauteur_cm",
                    models.IntegerField(default=30, verbose_name="hauteur en cm"),
                ),
                (
                    "categories",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=25,
                        verbose_name="Pour les categories",
                    ),
                ),
                (
                    "tronc",
                    models.BooleanField(default=False, verbose_name="Tronc commun"),
                ),
                (
                    "abrege",
                    models.CharField(
                        blank=True, default="", max_length=20, verbose_name="Abrégé"
                    ),
                ),
                (
                    "lib_option",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=20,
                        verbose_name="Option dans SIECLE",
                    ),
                ),
                (
                    "discipline",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.disciplines",
                        verbose_name="Discipline",
                    ),
                ),
                (
                    "filiere",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.filieres",
                        verbose_name="Filière",
                    ),
                ),
                (
                    "niveau",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.niveaux",
                        verbose_name="Niveau",
                    ),
                ),
                (
                    "serie",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.series",
                        verbose_name="Série",
                    ),
                ),
                (
                    "type_materiel",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.typemateriel",
                        verbose_name="Type de Matériel",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="inventaire",
            name="materiel",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="gestion.materiel",
            ),
        ),
        migrations.CreateModel(
            name="Details",
            fields=[
                ("id", models.IntegerField(primary_key=True, serialize=False)),
                ("Quantite", models.CharField(max_length=25, verbose_name="Quantité")),
                ("Remise", models.CharField(max_length=25, verbose_name="Remise")),
                (
                    "commande",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.commandes",
                    ),
                ),
                (
                    "materiel",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.materiel",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Details",
            },
        ),
        migrations.CreateModel(
            name="RevueStock",
            fields=[
                (
                    "code",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="gestion.inventaire",
                        verbose_name="N° de livre",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=datetime.date(1, 1, 1),
                        verbose_name="Date de la dernière vérification",
                    ),
                ),
                ("prete", models.BooleanField(default=False, verbose_name="Est prêté")),
                (
                    "dernier_pret",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.prets",
                        verbose_name="Dernier prêt",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Revue du stock",
            },
        ),
        migrations.CreateModel(
            name="LivreSupplement",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "data",
                    models.TextField(verbose_name="Demande de prêt supplémentaire"),
                ),
                ("date", models.DateTimeField(auto_now_add=True)),
                (
                    "commentaire",
                    models.TextField(blank=True, null=True, verbose_name="Motivation"),
                ),
            ],
            options={
                "verbose_name_plural": "Demandes de livres supplémentaires",
            },
        ),
        migrations.CreateModel(
            name="Livre_a_lire",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateTimeField(auto_now_add=True)),
                (
                    "eleve",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.eleves",
                        verbose_name="À prêter à",
                    ),
                ),
                (
                    "livre",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="gestion.materiel",
                        verbose_name="Livre",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Demandes de livres validées",
            },
        ),
        migrations.CreateModel(
            name="Courriers",
            fields=[
                (
                    "N",
                    models.IntegerField(
                        primary_key=True, serialize=False, verbose_name="N°"
                    ),
                ),
                (
                    "Pret_Recommandation",
                    models.TextField(verbose_name="Recommandation pour le prêt"),
                ),
                (
                    "Retour_Recommandation",
                    models.TextField(verbose_name="Recommandation pour le retour"),
                ),
                (
                    "Pret_Recom_Prof",
                    models.TextField(
                        verbose_name="Recommandation pour le prêt professeur"
                    ),
                ),
                (
                    "Retour_Recom_Prof",
                    models.TextField(
                        verbose_name="Recommandation pour le retour professeur"
                    ),
                ),
                (
                    "Memoire",
                    models.TextField(
                        default="",
                        verbose_name="Explications pour le «\xa0mémoire pour pertes\xa0»",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Textes prédéfinis des courriers",
            },
        ),
    ]



import re
import  csv
import  os
from difflib import SequenceMatcher

from common import safeText

from PyQt6.QtWidgets import QWidget, QCheckBox, QComboBox, QLineEdit, \
    QTableWidgetItem
from PyQt6.QtCore import Qt, pyqtSignal

from Ui_tableCSV import Ui_tableCSV
from Ui_unicity import Ui_unicity

class TableCSV(QWidget, Ui_tableCSV):
    """
    Une table pour interagir avec le contenu d'un fichier CSV

    Paramètres du constructeur
    @parant un tabWidget conteneurt
    @parant mw la fenêtre principale
    """

    # signaux spéciaux
    foreignCells = pyqtSignal(int, bool, list) # on modifie un combo de type
    fcChange = pyqtSignal(int) # on modifie une case Vers... et/ou alias
    
    # les numéros de colonnes
    NOTABLE_COL = 0            # pour les boutons radio de champ notable
    FIRST_EXAMPLE_COL = 1      # pour les trois exemples de valeur
    TYPE_COL = 4               # pour le combo de choix de type
    UNICITY_COL = 5            # pour les boutons radio d'unicité
    VERS_COL = 6               # pour les classes des clés "foreign"
    ALIAS_COL = 7              # alias si plusieurs "foreign" existent

    def __init__(self, parent, mw):
        QWidget.__init__(self, parent)
        Ui_tableCSV.__init__(self)
        self.setupUi(self)
        # connexion des signaux spéciaux
        self.foreignCells.connect(self.manageForeignCells)
        self.fcChange.connect(self.updateData)

        # intialisation diverses
        self.mw = mw         # pointeur vers la fenêtre principale
        self.fieldnames = [] # liste des noms de champs (vertical headers)
        self.filename = ""   # chemin absolu du fichier CSV
        self.modele = ""     # nom du modèle découlant de celui du fichier
        return

    def getConf(self, fieldname, key):
        """
        Récupère une valeur de la configuration
        @param fieldame un nom de champ
        @param key une de ses propriétés
        @return la valeur correspondant à la clé key dans
          self.mw.config["data_CSV"], ce qui converne le tableau local
        """
        if self.filename not in self.mw.config["data_CSV"]:
            self.mw.config["data_CSV"][self.filename] = {}
        if fieldname not in self.mw.config["data_CSV"][self.filename]:
            self.mw.config["data_CSV"][self.filename][fieldname] = {}
        return self.mw.config["data_CSV"][self.filename][fieldname].get(
            key, None)
    
    def populate(self, filename):
        """
        Remplit en grande part le tableau à partir d'un nom de fichier,
        et renseigne par effet de bord les données de configuration
        @param filename chemin absolu du fichier CSV
        """
        self.filename = filename
        dr = csv.DictReader(
            open(filename, "r", encoding="iso-8859-15"), delimiter = ",")
        self.modele = fn2modele(filename)
        self.fieldnames = dr.fieldnames # mémorise les noms de champs localement
        # recopie ces propriétés en les ajoutant à self.tableWidget
        self.tableWidget.filename = self.filename
        self.tableWidget.modele = self.modele
        self.tableWidget.fieldnames = self.fieldnames
        self.tableWidget.setRowCount(len(dr.fieldnames))
        self.tableWidget.setVerticalHeaderLabels(dr.fieldnames)
        # première colonne de faible largeur
        self.tableWidget.setColumnWidth (self.NOTABLE_COL, 30)
        # mise en place des exemples de données lues dans le fichier CSV
        for i in range(3):
            try:
                dic = next(dr)
                for j, (key,val) in enumerate(dic.items()):
                    self.tableWidget.setItem(
                        j, i+self.FIRST_EXAMPLE_COL, QTableWidgetItem(val))
            except StopIteration:
                # s'il y a peu d'exemples, on laisse tomber
                break
        # mise en place de la case à cocher "notable", selon la
        # configuration, sinon cochée s'il y a déjà quelques exemples.
        for j, fn in enumerate(dr.fieldnames):
            notableWidget = QCheckBox("", self)
            isNotable = self.getConf(fn, "notable") == "true"
            checked = Qt.CheckState.Unchecked
            if isNotable:
                checked = Qt.CheckState.Checked
            elif self.getConf(fn, "notable") is None:
                # rien dans la config ...
                # faut-il cocher la case notable quand même ?
                # on regarde si on voit des données parmi les exemples
                notableVal = "false"
                for i in range(3):
                    cell = self.tableWidget.item(j, i+self.FIRST_EXAMPLE_COL)
                    if cell and cell.text():
                        checked = Qt.CheckState.Checked
                        notableVal = "true"
                        break
                self.mw.updateConfig.emit(
                    self.filename, fn, "notable", notableVal)
            notableWidget.setCheckState(checked)
            notableWidget.stateChanged.connect(lambda: self.mw.toDjango.emit())
            self.tableWidget.setCellWidget(j, self.NOTABLE_COL, notableWidget)
        # mise en place des combos pour le choix du type
        for j, fn in enumerate(dr.fieldnames):
            self.tableWidget.setCellWidget(
                j, self.TYPE_COL, TypeWidget(self, j, fn))
        # mise en place des boutons radio d'unicité
        for j, fieldname in enumerate(dr.fieldnames):
            self.tableWidget.setCellWidget(
                j, self.UNICITY_COL, UnicityWidget(self, fieldname))
        return

    def manageForeignCells(self, r, add = True, modeles = []):
        """
        Ajoute des cases pour les clés étrangères dans la ligne numéro r
        du tableau
        @param r Le numéro de ligne
        @param add Vrai [par défaut] quand on veut ajouter les cases
          dans le tableau. Faux quand on veut les supprimer
        @param modeles Une liste des modèles possibles à choisir, à préciser
          quand le paramètre add est vrai.
        """
        t = self.tableWidget
        if add:
            # mise en place du combo en colonne self.VERS_COL
            fieldname = self.fieldnames[r]
            # ajoute un combo de modèles
            modelW = ModelWidget(t, fieldname, modeles)
            modelW.currentIndexChanged.connect(lambda: self.fcChange.emit(r))
            t.setCellWidget(r, self.VERS_COL, modelW)
            cell = t.cellWidget(r, self.VERS_COL)
            foreign = self.getConf(fieldname, "foreign")
            if foreign:
                cell.setCurrentText(foreign)
                self.fcChange.emit(r) # ??? pourquoi ?
            else:
                # c'est un champ encore inconnu de la configuration
                # fait en sorte de deviner le meilleur modèle
                # étant donné le nom du champ
                fn = safeText(fieldname).lower()
                scores = {
                    m : SequenceMatcher(
                        None,
                        safeText(m).lower(), fn).find_longest_match().size for \
                    m in (cell.itemText(i) for i in range(cell.count()))
                }
                # s'il y a au moins 4 lettres en commun, on change de modèle
                best = max(scores.items(), key = lambda x: x[1])
                if best[1] > 3:
                    cell.setCurrentText(best[0])
                    self.mw.updateConfig.emit(
                        self.filename, fieldname, "foreign", best[0])
            # mise en place du lineEdit en colonne self.ALIAS_COL
            edit = QLineEdit("")
            edit.editingFinished.connect(lambda: self.fcChange.emit(r))
            edit.setPlaceholderText("alias ?")
            alias = self.getConf(fieldname, "alias")
            if alias:
                edit.setText(alias)
                self.fcChange.emit(r) # ??? pourquoi ?
            t.setCellWidget(r, self.ALIAS_COL, edit)
        else: # add est faux, on supprimes les cases
            t.removeCellWidget(r, self.VERS_COL)
            t.removeCellWidget(r, self.ALIAS_COL)
        return

    def updateData(self, r):
        """
        Mise à jour de la portion de configuration pointée par
        self.data pour prendre en compte le contenu des colonnes
        self.VERS_COL et self.ALIAS_COL. Cette fonction de rappel
        est connectée au signal self.fcChange.

        @param r la ligne du tableau où un changement a eu lieu
        """
        fieldname = self.fieldnames[r]
        c = self.tableWidget.cellWidget(r, self.VERS_COL)
        a = self.tableWidget.cellWidget(r, self.ALIAS_COL)
        if c:
            self.mw.updateConfig.emit(
                self.filename, fieldname, "foreign", c.currentText())
        if a:
            self.mw.updateConfig.emit(
                self.filename, fieldname, "alias", a.text())
        return

class TypeWidget(QComboBox):
    """
    Un widget pour choisir le type de champ. Les types possibles sont
    propres à la classe MaFenetre

    paramètres du constructeur:
    @param parent le widget parent
    @param row la ligne de tableau dans le widget parent
    @param fieldname le nom du champ pour la ligne courante ; il y a un
      traitement spécial quand la configuration est indéfinie et que
      le nom de champ commence par un préfixe spécial comme "Clé" ou "Id_"
    """

    prefixes_speciaux = {
        "cle" : "foreign",
        "id_"  : "int",
    }
    
    def __init__(self, parent, row, fieldname):
        QComboBox.__init__(self, parent)
        self.row = row
        self.mw = parent.mw
        self.parent = parent
        self.fieldname = fieldname
        self.addItems(self.mw.types.keys())
        parent.mw.registeredCombos[fieldname] = self
        atype = self.parent.getConf(fieldname, "type")
        if atype:
            self.setCurrentText(atype)
        else:
            # si le nom de champ commence par un préfixe spécial ...
            for prefixe, preference in self.prefixes_speciaux.items():
                if fieldname.lower().startswith(prefixe):
                    self.setCurrentText(preference)
        self.currentIndexChanged.connect(self.changed)
        return

    def changed(self):
        """
        fonction de rappel pour le changement de l'index courant
        """
        atype = self.currentText()
        if atype == "foreign":
            self.parent.foreignCells.emit(self.row, True, self.mw.modeles)
        else:
            self.parent.foreignCells.emit(self.row, False, [])
        self.mw.toDjango.emit()
        return
    

class UnicityWidget(QWidget,Ui_unicity):
    """
    Un widget avec des bouton radio pour clé primaire et unique.

    Paramètres du constructeur:
    @param parent le widget parent
    @param fieldname un nom de champ ; s'il commence par "Id_", le bouton
      de clé primaire sera coché
    """
    def __init__(self, parent, fieldname):
        QWidget.__init__(self, parent)
        Ui_unicity.__init__(self)
        self.setupUi(self)
        unicity = parent.getConf(fieldname, "unicity")
        if unicity == "primary_key":
            self.primButton.setChecked(True)
        elif unicity == "unique":
            self.uniqueButton.setChecked(True)
        else:
            # pas encore de config pour "unicity" ?
            # on regarde quand même si le nom de champ évoque une clé primaire
            if fieldname.lower().startswith("id_"):
                self.primButton.setChecked(True)
                parent.mw.updateConfig.emit(
                    parent.filename, fieldname, "unicity", "primary_key")
            else:
                self.radioButton.setChecked(True)
        # on connecte les trois boutons radio à la modification du code
        for button in (self.primButton, self.uniqueButton, self.radioButton):
            button.toggled.connect(lambda: parent.mw.toDjango.emit())
        return

    def __str__(self):
        if self.primButton.isChecked():
            return "primary_key"
        if self.uniqueButton.isChecked():
            return "unique"
        return "none"

    @property
    def isPrimaryKey(self):
        """
        @return Vrai si le bouton Clé primaire est coché
        """
        return self.primButton.isChecked()
    
    @property
    def isUnique(self):
        """
        @return Vrai si le bouton Unique est coché
        """
        return self.uniqueButton.isChecked()

    @property
    def djangoParam(self):
        """
        @return une mention pour un champ de modèle
        """
        if self.isPrimaryKey:
            return ", primary_key = True"
        elif self.isUnique:
            return ", unique = True"
        return ""
    
class ModelWidget(QComboBox):
    """
    Un widget pour choisir le modèle cible quand un champ est de type foreign

    paramètres du constructeur:
    @param parent le widget parent
    @param fieldname le nom du champ pour la ligne courante ; il y a un
      traitement spécial quand la configuration est indéfinie et que
      le nom de champ commence par un préfixe spécial comme "Clé" ou "Id"
    @param modeles une liste des modèles possibles
    """

    def __init__(self, parent, fieldname, modeles):
        QComboBox.__init__(self, parent)
        self.addItems(["Autre_Modele"] + list(modeles))
        return
    
def fn2modele(filename):
    """
    Déduit un nom de modèle du nom d'un fichier
    """
    return re.match(r".*/(.*)\.csv", filename).group(1)

def modele2fn(dir, modele):
    """
    Déduit un nom de fichier d'un nom de modèle
    """
    return os.path.join(dir, modele + ".csv")





from pyx import path, canvas, style, deco, text, color, connector, document, \
    svgwriter
from svg.path import parse_path
from subprocess import call
import tempfile, os
from PIL import Image
from io import BytesIO
from cairosvg import svg2png
import xml.etree.ElementTree as ET
import re

class TextArrow:
    """
    Crée un texte sur un fond de couleur de largeur donnée,
    attaché à une flèche
    Les paramètres du constructeur sont :
    @param xt abscisse du centre du texte
    @param yt ordonnée du centre du texte
    @param wt largeur max. de la boîte de texte
    @param x2 abscisse du bout de la flèche
    @param y2 ordonnée du bout de la flèche
    @param atext un texte (unicode)
    @param height_minus (None par défaut); s'il est défini, les ordonnees
        sont calculées comme height_minus - y
    """
    
    def __init__(self, xt, yt, wt, x2, y2, atext, height_minus=None):
        self.xt = float(xt)
        self.yt = float(yt)
        self.wt = float(wt)
        self.x2 = float(x2)
        self.y2 = float(y2)
        self.atext = atext
        if height_minus is not None:
            height_minus = float(height_minus)
            self.yt = height_minus - self.yt
            self.y2 = height_minus - self.y2
        self.attrs = [text.halign.center, text.vshift.middlezero]
        return

    def to_python(self):
        """
        produit la source python pour créer un clone de l'instance
        avec la possibilité de traduire le texte
        """
        return """\
TextArrow({xt:.1f}, {yt:.1f}, {wt:.1f}, {x2:.1f}, {y2:.1f},
    _('''{atext}'''))\
""".format(**self.__dict__)

    def __str__(self):
        return "« {atext} » : position, largeur = ({xt}, {yt}), {wt} ; pointe vers : ({x2}, {y2})".format(**self.__dict__)
    
    @staticmethod
    def from_SVG(source):
        """
        Crée une liste d'instances de TextArrow étant donné
        un fichier SVG
        @param source un nom de fichier, ou un flux de données
        """
        ns = {
            "inkscape": "http://www.inkscape.org/namespaces/inkscape",
            "sodipodi": "http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd",
            "xlink": "http://www.w3.org/1999/xlink",
            "svg": "http://www.w3.org/2000/svg",
        }
        root = ET.parse(source).getroot()
        result=[]
        for group in root.findall(".//svg:g", ns):
            text = group.find("./svg:text", ns)
            has_text = isinstance(text, ET.Element)
            path = group.find("./svg:path", ns)
            has_path = isinstance(path, ET.Element)
            rect = group.find("./svg:rect", ns)
            has_rect = isinstance(rect, ET.Element)
            if has_text and has_path and has_rect:
                letexte = " ".join([t. text for t in text.findall("./svg:tspan", ns)])
                xt = text.attrib.get('x')
                yt = text.attrib.get('y')
                wt = rect.attrib.get('width')
                lapointe = parse_path(path.attrib.get("d"))
                z = lapointe[-1].end
                x2, y2 = z.real, z.imag
                result.append(TextArrow(xt, yt, wt, x2, y2, letexte, height_minus = root.attrib.get("height")))
        return result
        

def annote(img, *textarrows):
    """
    Annote une image "en place" si c'est une instance de PIL.Image.Image,
    mais accepte aussi un nom de fichier.
    @param img un nom de fichier ou une instance de PIL.Image.Image
    @param textarrows une liste d'instances de TextArrow
    @return une instance de PIL.Image.Image
    """
    if not isinstance(img, Image.Image):
        img = Image.open(img)
    img.convert("RGBA")
    # calcul d'échelle pour que les tailles des annotations et des flèches
    # soient raisonnables par rapport à l'image bitmap
    scale = 18/max(img.width, img.height)
    # LaTex devra accepter du texte unicode
    text.set(engine=text.LatexRunner, texenc='utf-8')
    text.preamble(r'\usepackage{ucs}')
    text.preamble(r'\usepackage[utf8x]{inputenc}')
    frame = path.rect(
        0, 0, img.width*scale, img.height*scale)
    c = canvas.canvas()
    c.draw(frame, []) # rectangle englobant, invisible
    backgroundstyle = [style.linewidth.Thick, color.rgb.blue,
                       deco.filled([color.rgb.white])]
    curvestyle = [color.rgb.blue, deco.earrow.normal, style.linewidth.Thick]
    for t in textarrows:
        # création d'objets à dessiner
        textstyle = [text.parbox(t.wt*scale)] + t.attrs
        text_ = text.text(t.xt*scale, t.yt*scale, t.atext, textstyle)
        background = text_.bbox().enlarged(0.1).path()
        curve = connector.curve(
            text_, text.text(t.x2*scale, t.y2*scale, " "), boxdists=[0.1, 0.1])
        # dessin sur le canevas
        c.stroke(background, backgroundstyle)
        c.insert(text_)
        c.stroke(curve, curvestyle)
    # export et conversion au format PNG lisible par PIL.Image
    SVGbuffer = BytesIO()
    PNGbuffer = BytesIO()
    svgwriter.SVGwriter(document.document([document.page(c)]), SVGbuffer)
    SVGbuffer.seek(0)
    svg2png(
        file_obj = SVGbuffer,
        output_width = img.width,
        output_height = img.height,
        write_to = PNGbuffer)
    PNGbuffer.seek(0)
    # récupération des annotations et surimpression sur l'image
    annotations = Image.open(PNGbuffer, formats=["png"])
    img.paste(annotations, (0,0), annotations)
    return img

def annote_par_svg(PNGimage,SVGfilename):
    """
    crée une instance de PIL.Image.Image, à partir d'un fichier PNG
    et d'un fichier SVG ; ce dernier doit contenir des groupes, chacun
    contenant un texte, un rectangle et une flèche. Chaque groupe forme
    une annotation, et la taille du fichier SVG doit correspondre à celle
    de l'image PNG (en pixels)
    @param PNGimage nom d'un fichier d'image au format PNG, ou instance
           de PIL.Image.Image
    @param SVGfilename nom d'un fichier d'image au format SVG
    @return une instance de PIL.Image.Image
    """
    if type(PNGimage) is str:
        PNGimage = Image.open(PNGimage)
    return annote(PNGimage, *TextArrow.from_SVG(SVGfilename))

def to_python_avec_svg(SVGfilename):
    """
    crée une source en Python qui fait en partie le travail de la fonction
    précédente : la liste TextArrow.from_SVG(SVGfilename)
    @param PNGfilename nom d'un fichier d'image au format PNG
    @param SVGfilename nom d'un fichier d'image au format SVG
    @return une instance de PIL.Image.Image
    """
    tas = TextArrow.from_SVG(SVGfilename)
    return "[\n" + ",\n".join(["  "+ ta.to_python() for ta in tas]) + "\n]"

def demo():
    source = Image.open("content/images/fr-fr/cartemembre4.png")
    img = annote_par_svg(source, "annotations/cartemembre4.svg")
    img.save("/tmp/resultat.png")
    print("L'annotation est ajoutée, dans /tmp/resultat.png")
    print("==== un peu de code source Python ====")
    print(to_python_avec_svg("annotations/cartemembre4.svg"))
    
if __name__ == "__main__":
    demo()
    



"""
    gestion.views_common, un module pour SLM
    - partie commune du rendu pages web à l'aide de modèles

    Copyright (C) 2023 Georges Khaznadar <georgesk@debian.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from django.utils.translation import activate, get_language_info

from .version import this_version
from manuels.settings import language_for

import unicodedata

class Onglet:
    """
    Cette classe définit les données d'un onglet. Les paramètres du
    constructeur sont:
    @param title le titre de l'onglet
    @param page le nom de da fonction à invoquer dans le module views
    @param pour_tous vrai si l'onglet est défini pour tous (faux par défaut)
    """
    def __init__(self, title, page, pour_tous = False):
        self.title = title
        self.page = page
        self.pour_tous = pour_tous
        return
    
Les_onglets = [ ## accès aux pages de niveau 1
    Onglet(_("Paramètres"), "parametres"),
    Onglet(_("Achats/Inventaire"), "achats_inventaire"),
    Onglet(_("Prêts"), "prets"),
    Onglet(_("Retours"), "retours"),
    Onglet(_("Boutique"), "eleve_invite"),
    Onglet(_("Assistance"), "assistance"),
    Onglet(_("À propos"), "apropos", pour_tous = True),
]


def render_common(request, template, subtitle="", context=None,
                  content_type=None, status=None, using=None,
                  pour_tous=False):
    """
    rendu des pages incluant des onglets, basé sur django.shortcuts.render
    @param request l'objet requête
    @param template le modèle de page à rendre
    @param subtitle un sous-titre spécifique à la page à rendre
    @param context le contexte pour utiliser le modèle ; il sera enrichi
      pour produire le titre et les onglets
    @param content_type idem que pour django.shortcuts.render
    @param status idem que pour django.shortcuts.render
    @param using idem que pour django.shortcuts.render
    @param pour_tous permet de forcer le caractère public de la page ;
       si la page n'est pas publique, un message demande de s'authentifier
       (faux par défaut)
    @return un objet de type HTTPRequest
    """
    lalangue = request.session.get("lang", "en") # en par défaut
    activate(lalangue)
    dico = {
        "version": this_version,
        "nom_page": subtitle,
        "onglets": Les_onglets,
        "page_pour_tous": pour_tous,
        "lalangue": lalangue,
        "locale": language_for(lalangue),
    }
    if context is None:
        context = dico
    else:
        context.update(dico)
    if "aide" not in dico:
        dico["aide"] = True
    return render(request, template, context, content_type, status, using)
    



#! /usr/bin/python3

import sys
import  os
import  re
import  yaml
from pathlib import Path
from datetime import datetime
from copy import copy

from PyQt6.QtWidgets import QApplication, QMainWindow, QFileDialog, QMessageBox

from PyQt6.QtCore import pyqtSignal, Qt

from tableCSV import TableCSV, fn2modele, modele2fn
from common import safeIdent

from Ui_csv2modele import Ui_MainWindow

License = """\
Copyright (c) 2023 Georges Khaznadar <georgesk@debian.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
"""


CONFIG_FILE = os.path.join(Path.home(), ".local", "share", "csv2modele", "config.yaml")


DEBUT_CLASSE = '''\
class {nom}(models.Model):
    """
    Ici, la documentation de la classe {nom}
    """
'''


class MaFenetre(QMainWindow, Ui_MainWindow):

    toDjango = pyqtSignal()                  # tableaux => code de models.py
    updateConfig = pyqtSignal(str, str, str, str) # mise à jour de self.config

    registeredCombos = {}

    types = {
        "text": "CharField(max_length=25, {vn}{unicity})",
        "int" : "IntegerField({vn}{unicity})",
        "float": "DecimalField(max_digits=6, decimal_places=2, {vn}{unicity})",
        "date": "DateField({vn}{unicity})",
        "foreign": "models.ForeignKey('{foreign}'{related_name}, " + \
                   "on_delete=models.CASCADE)"
    }
    
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)
        Ui_MainWindow.__init__(self)
        self.setupUi(self)
        self.config=self.readConfig()

        self.filenames = []                   # fichiers CSV ouverts
        self.modeles = []                     # noms de modèles, accentués
        self.file_tableCSV = {}               # dict. fichier -> tableCSV
        self.tableWidgets = []                # liste des tablewidgets
        self.dir_csv = self.config["dir_csv"] # répertoire des fichiers CSV
        self.export_to_model = \
            self.config["export_to_model"]    # répertoire pour models.py
        self.tabWidget.setCurrentIndex(0)
        self.modeles_edit.setText(self.export_to_model)
        self.toDjango.connect(self.csv2django)
        self.updateConfig.connect(self.update_data_CSV)
        self.dir_csv_button.clicked.connect(self.selectDir)
        self.modeles_button.clicked.connect(self.selectExportDir)
        self.add2modeles_button.clicked.connect(self.exportModeles)
        if len(sys.argv) > 1:
            self.selDirDonne(os.path.abspath(sys.argv[1]))
        elif self.csv_files_in_dir():
            self.selDirDonne(os.path.abspath(self.dir_csv))
        return

    def exportModeles(self):
        """
        Exporte les sources Python vers le fichier models.py du répertoire
        self.export_to_model
        """
        path = os.path.join(self.export_to_model, "models.py")
        ok = not os.path.exists(path)
        if not ok:
            reply = QMessageBox.question(
                self, "Fichier models.py", """\
Un fichier models.py existe déjà
dans le répertoire d'exportation.
OK pour réécrire par-dessus ?""", )
            ok = bool(reply)
        if ok:
            with open(path, "w") as outfile:
                outfile.write(self.pythonEditor.text())
        return
    
    def selectExportDir(self):
        """
        Met à jour self.export_to_model
        """
        dir_ = QFileDialog.getExistingDirectory(
            self, "Répertoire des fichiers CSV", self.export_to_model)
        if dir_:
            self.export_to_model = dir_
            self.config["export_to_model"] = dir_
            self.modeles_edit.setText(dir_)
        return
    
    def selectDir(self):
        """
        Met à jour self.dir_csv puis ouvre les fichiers CSV présents dans
        le répertoire dans autant de tabs de self.tabFichiers
        """
        dir_ = QFileDialog.getExistingDirectory(
            self, "Répertoire des fichiers CSV", self.dir_csv)
        if dir_:
            self.dir_csv = dir_
            self.selDirDonne(dir_)
        return

    def selDirDonne(self, dir_):
        """
        Sélectionne un répertoire donné pour les fichiers CSV, puis
        peple les onglets relatifs à chacun des fichiers ; enfin, exporte
        une fois en code Python pour Django.
        
        @param dir_ le répertoire donné
        """
        self.config["dir_csv"] = dir_
        self.dir_csv_edit.setText(dir_)
        # on commence par vider self.tabFichiers
        for i in range(self.tabFichiers.count(), 0, -1):
            w = self.tabFichiers.widget(i-1)
            w.close()
            del (w)
            self.tabFichiers.removeTab(i-1)
        # puis on ajoute un nouveau tab par fichier CSV de self.dir_csv
        self.filenames = [
            os.path.join(self.dir_csv, f.name) for f in self.csv_files_in_dir()]
        self.modeles = [fn2modele(fn) for fn in self.filenames]
        for f in self.filenames:
            # on crée le tableau CSV
            w = TableCSV(self.tabFichiers, self)
            label = re.sub(".csv$", "", os.path.basename(f))
            # on insère ce tableau dans un nouvel onglet
            self.tabFichiers.addTab(w, label)
            self.file_tableCSV[f] = w
            self.tableWidgets.append(w.tableWidget)
            # et là on y déploie le fichier
            self.ouvreFichierDonne(os.path.join(self.dir_csv, f), w)
        # met en place la colonne "Vers..."
        for t in self.tableWidgets:
            for r in range(t.rowCount()):
                if t.cellWidget(
                        r, TableCSV.TYPE_COL).currentText() == "foreign":
                    t.parent().foreignCells.emit(r, True, self.modeles)
        # lance csv2django une première fois
        self.toDjango.emit()
        return

    def csv_files_in_dir(self):
        """
        @return la liste pathlib.PosixPath (CSV) trouvés dans self.dir_csv
        """
        return (f for f in sorted(Path(self.dir_csv).iterdir()) if \
                f.is_file() and f.name.endswith(".csv"))
    
    def readConfig(self):
        """
        Récupère la configuration
        @return : la configuration
        """
        if not os.path.exists(CONFIG_FILE):
            # création d'une configuration vierge
            os.makedirs(os.path.dirname(CONFIG_FILE), exist_ok=True)
            config = {
                "known_CSV": [],        # chemins absolus de fichiers déjà vus
                "data_CSV": {},         # dict. fichiers -> détails des fichiers
                "dir_csv":  ".",        # répertoire des fichiers CSV
                "export_to_model": '.', # chemin vers le répertoire de models.py
                "modele_dep": {},       # dépendances entre modèles
            }
            with open(CONFIG_FILE, "w") as yamlfile:
                yaml.dump(config, yamlfile)
        return yaml.safe_load(open(CONFIG_FILE))

    def saveConfig(self):
        """
        Enregistre la configuration
        """
        with open(CONFIG_FILE, "w") as yamlfile:
            yaml.dump(self.config, yamlfile)
        return

    
    def closeEvent(self, ev):
        """
        Calcule la dépendance entre modèles, ... puis
        registre la configuration avant de fermer
        """
        self.saveConfig()
        return

    def update_modele_dep(self):
        """
        Calcul des dépendances ente modèles et inscription
        dans la configuration (self.config["modele_dep"])
        """
        for m in self.modeles:
            filename = modele2fn(self.dir_csv, m)
            data = self.config["data_CSV"][filename]
            foreigns = [data[fn].get("foreign") for fn in data if \
                        'type' in data[fn] and data[fn]['type'] == 'foreign']
            self.config["modele_dep"][m] = foreigns
        return
    
    def ouvreFichierDonne(self, filename, w):
        """
        ouvre un fichier de données et le déploie dans un tableau interactif
        @param filename le nom du fichier à ouvrir
        @param w une instance de TableCSV (qui a une propriété .tableWidget)
        """
        filename = os.path.abspath(filename)
        # Il faudra changer ailleurs toutes les dates
        # au format jj/mm/aaaa en format aaaa-mm-jj, le seul qui soit
        # lu directement par Django.
        #call(f"./fixTheDates.sh '{filename}'", shell = True)
        if filename not in self.config["known_CSV"]:
            self.config["known_CSV"].append(filename)
            self.config["data_CSV"][filename] = {} # dict. des données fichier
        # voici les données de configuration pour le fichier courant
        w.populate(filename)
        return

    def update_data_CSV(self, filename, fieldname, key, val):
        """
        Mise à jour de self.config["data_CSV"][filename][fieldname][key],
        avec la valeur val ; fonction de rappel pour le signal
        updateConfig
        """
        if filename not in self.config["data_CSV"]:
            print (f"""Création de self.config["data_CSV"][{filename}]""")
            self.config["data_CSV"][filename] = {}
        if fieldname not in self.config["data_CSV"][filename]:
            print (f"""Création de self.config["data_CSV"][{filename}][{fieldname}]""")
            self.config["data_CSV"][filename][fieldname] = {}
        self.config["data_CSV"][filename][fieldname][key] = val
        return

    def verboseName(self, fieldname):
        """
        @return une mention à la bonne syntaxe pour Django
        """
        return '''verbose_name = """{}""" '''.format(fieldname)

    def notables(self, table):
        """
        Écrit du code pour la fonction __str__ qui décrit un modèle
        selon les cases de notabilité cochées
        @param table le tableau à considérer
        @return une chaîne à formater de type f""
        """
        notableFields = [
            safeIdent(fn) for j, fn in enumerate(table.fieldnames) \
            if table.cellWidget(j, TableCSV.NOTABLE_COL).checkState() \
            == Qt.CheckState.Checked
        ]
        return "{self." + "} {self.".join(notableFields) + "}"
    
    def csv2django(self):
        """
        Produit du code Python, utile pour Django, dans le deuxième volet,
        et par effet de bord, modifie aussi la configuration.
        """
        # recalcule les dépendances
        self.update_modele_dep()
        # mise en ordre pour respecter les dépendances
        max = len(self.filenames) # pas plus de max tours de boucle:
        orderedfiles = []
        n = 0
        not_ordered = copy(self.filenames)
        while not_ordered:
            oset = set((fn2modele(f) for f in orderedfiles))
            for f in not_ordered:
                print("GRRR on teste", fn2modele(f))
                dependencies = set(self.config["modele_dep"][fn2modele(f)])
                if oset.issuperset(dependencies) and f not in orderedfiles:
                    orderedfiles.append(f)
                    i = not_ordered.index(f)
                    del not_ordered[i]
                    print("GRRR on ajoute", fn2modele(f), "n =", n)
            n += 1
            if n > max:
                QMessageBox.critical(
                    self,
                    "ERREUR",
                    """\
Il y a une dépendance circulaire,
revoyez les colonnes Vers...""")
                return
            
        # orderedfiles contient une liste de fichiers tels que les
        # dépendances soient satisfaites si on le traite sans cet ordre
        
        result = MODELS_HEAD # écrit l'en-tête du fichier models.py

        # on marque ici les modèles dans l'ordre des dépendances
        result += f"modeles_ordre = {[fn2modele(f) for f in orderedfiles]}\n\n"

        for f in orderedfiles:
            w = self.file_tableCSV[f]
            # Creation de la classe
            nom = safeIdent(re.sub(r"\.csv$", "", os.path.basename(f)))
            result += DEBUT_CLASSE.format(nom = nom)

            # Fonction d'accès au widget dans une case de tableau
            at = w.tableWidget.cellWidget
            # définit les champs comme des propriétés de cette classe
            for r in range(w.tableWidget.rowCount()):
                # écrit la ligne de code pour le champ "key" de la ligne courante
                key = w.tableWidget.verticalHeaderItem(r).text()
                model = self.types[self.registeredCombos[key].currentText()]
                unicityCell = at(r, TableCSV.UNICITY_COL)
                unicity = unicityCell.djangoParam if unicityCell else None
                foreignCell = at(r, TableCSV.VERS_COL)
                foreign = safeIdent(foreignCell.currentText()) if \
                    foreignCell else None
                aliasCell = at(r, TableCSV.ALIAS_COL)
                alias = ", related_name='{}'".format(aliasCell.text()) if \
                    aliasCell and aliasCell.text() else ""
                result += "    {key} = {modelBuilder}\n".format(
                    key = safeIdent(key),
                    modelBuilder = model.format(
                        vn = self.verboseName(key),
                        unicity = unicity,
                        foreign = foreign,
                        related_name = alias,
                    ))
                # met à jour la configuration
                isNotable = at(r, TableCSV.NOTABLE_COL).checkState() == \
                    Qt.CheckState.Checked
                self.updateConfig.emit(
                    f, key, "notable", "true" if isNotable else "false")
                self.updateConfig.emit(
                    f, key, "unicity", str(at(r, TableCSV.UNICITY_COL)))
                self.updateConfig.emit(
                    f, key, "type", self.registeredCombos[key].currentText())
            # crée la fonction __str__ du modèle à partir des champs notables
            result += "\n"
            result += "    def __str__(self):\n"
            result += "        return f'''" + self.notables(w.tableWidget) + "'''\n\n"

            # enregistre le modèle et son fichier CSV associé
            t= w.tableWidget
            fieldtypes = {safeIdent(t.verticalHeaderItem(i).text()): \
                          valType(t, i) \
                          for i in range(t.rowCount())}
            result += f"""\
les_modeles['{nom}'] = {{
    'csvfile': '{f}',
    'fieldtypes': {fieldtypes},
}}

"""
        self.pythonEditor.setText(result)
        self.statusBar().showMessage("Écriture de code Django ...", 2000)
        return

# début du fichier models.py
MODELS_HEAD = """\
# This file was generated automatically at {datetime}
# by the program csv2modele.py
""".format(datetime = datetime.now().strftime("%Y-%m-%d (%H:%M)"))

MODELS_HEAD +="""\
'''
{License}
'''
""".format(License = License)

MODELS_HEAD += """\
from django.db import models
from django.db.models import *
from django.utils import timezone
import sys, time, re, math
from datetime import timedelta, date, datetime
from copy import copy
import random

DATE111 = date.fromisoformat("0001-01-01")

les_modeles = {} # pour enregistrer les modèles

def preprocess(val, typedic):
    '''
    Prépare une valeur pour l'utiliser dans la création d'une instance
    de modèle ; ça permet de traiter le cas des chaînes vides qui ne donnent
    pas facilement des entiers, des dates, etc.
    @param val une valeur
    @param typedic un dictionnaire : nom de type -> valeur supplémentaire
    '''
    typestr, supp = list(typedic.items())[0] # seul élément de typedic
    if typestr == "int" and val =="": return "-999999999"
    if typestr == "float" and val =="": return "-999999999.0"
    if typestr == "date" and val =="": return "0001-01-01"
    if typestr == "foreign":
        try:
            result = eval(
              "{supp}.objects.get(pk={val})".format(supp=supp, val=int(val)))
        except:
            result = "ERREUR: Impossible de trouver une instance de {supp} " + \\
                     "avec une clé {val}".format(supp=supp, val=int(val))
        return result
    # par défaut, on ne change rien à val
    return val

verbose = False # à modifier pour avoir plus de trace dans le terminal

def log(message):
    '''
    Enregistre et date un message d'erreur
    '''
    with open("fromCSV.log", "a") as errorfile:
        errorfile.write("{now)}: {message}\n".format(
            now=datetime.now(), message=message))
    if verbose:
        print(message)
    return

def fromCSV(model, filedict):
    '''
    Enrichit la base de données à partir d'un fichier CSV
    @param model un modèle (à voir ci-dessous)
    @param filedict dictionnaire:
      'csvfile'    -> le fichier source du modèle
      'fieldtypes' -> dictionnaire champ -> type du champ
    '''
    filename = filedict['csvfile']
    fieldtypes = filedict['fieldtypes']
    print(f"Importation des données de {filename}")
    max= len(open(filename).readlines()) - 1
    import csv
    dr = csv.DictReader(open(filename), delimiter=",")
    idents = {fn: safeIdent(fn) for fn in dr.fieldnames}
    i = 1
    t0 = time.time()
    for dict in dr:
        dict1 = {idents[fn]: preprocess(val, fieldtypes[idents[fn]]) for \
            fn, val in dict.items()}
        if verbose: print(dict1)
        erreur = {
          ident: val for ident, val in dict1.items() \\
                 if isinstance(val, str) and \\
                 val.startswith("ERREUR: Impossible de trouver une instance de")
        }
        if erreur:
            log(f"{filename} ligne {i+1} : {erreur}")
        else:
            record = model.__class__(**dict1)
            record.save()
        if i%10 == 0:
            t1 = time.time()
            attente = round((max - i) *  (t1-t0) / i)
            sys.stdout.write( f"{i}/{max}      EOT: {attente} s  " + \\
                              f"DUR: {round(t1-t0)} s\\r")
            sys.stdout.flush()
        i += 1
    print()
    print(f"Importation des données de {filename} terminée")
    return

def safeText(text):
    '''
    routine de désaccentuation
    '''
    result=text
    desaccent = {
        r"[àâä]" : "a",
        r"[ÀÂÄ]" : "A",
        r"[éèêë]" : "e",
        r"[ÉÈÊË]" : "E",
        r"[îï]" : "i",
        r"[ÎÏ]" : "I",
        r"[ôö]" : "o",
        r"[ÔÖ]" : "O",
        r"[ùûü]" : "u",
        r"[ÙÛÜ]" : "U",
        r"[ç]" : "c",
        r"[Ç]" : "C",
    }
    for r, l in desaccent.items():
        result = re.sub(r,l, result)
    return result
    
def safeIdent(text):
    '''
    nettoyage des identifiants
    '''
    return re.sub(r'''[. ,;!"'?]''', "_", safeText(text).replace("°", ""))

# Create your models here.

"""

def valType(t, i):
    """
    Renvoie un dictionnaire type -> information supplémentaire
    C'est utile pour ce qui est des clés étrangères, car il faut la classe
    de la clé
    """
    atype = t.cellWidget(i, TableCSV.TYPE_COL).currentText()
    if atype == "foreign":
        supp = safeIdent(t.cellWidget(i, TableCSV.VERS_COL).currentText())
    else:
        supp = None
    return {atype: supp}
    
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = MaFenetre()
    w.show()
    sys.exit(app.exec())



"""
    template_i18n_strings.py
    - fouille les templates de django et y cherche les fichiers
      de type HTML ou RML, isole le chaînes (pas les tags) et les
      prépare à la façon de : {% translate la chaîne à traduire %}

    Copyright (C) 2024 Georges Khaznadar <georgesk@debian.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import bs4
from bs4 import BeautifulSoup as BS
import sys, os, re
import ply.lex as lex

def markup(s, start='{% translate "', end='" %}'):
    """
    marquage d'un texte, par mise en place de deux marqueurs :
    le marqueur de fin vient avant les espaces terminaux, le
    marqueur de début après les expaces initiaux
    @param s chaîne à marquer
    @param start marqueur de début : "{% translate " par défaut
    @param end marquetr de fin : " %}" par défaut
    @return la chaîne marquée
    """
    if not s.strip():
        return s
    pos= len(s)-1
    while s[pos] in " \t\n":
        pos -= 1
    s = s[:pos+1] + end + s[pos+1:]
    pos = 0
    while s[pos] in " \t\n":
        pos += 1
    s = s[:pos] + start + s[pos:]
    return s

class TransLexer(object):
    tokens = (
        "TEXT",
        "VAR",
        "CTRL",
    )

    states = (
       ('var','exclusive'),     # not to translate
       ('ctrl','exclusive'),     # not to translate
    )

    def t_begin_var(self,t):
        r'\{\{'
        t.lexer.begin('var')
        return

    def t_var_end_var(self,t):
        r'\}\}'
        t.lexer.begin('INITIAL')
        return

    t_var_VAR = r'[^\}]+'

    def t_begin_ctrl(self,t):
        r'\{%'
        t.lexer.begin('ctrl')
        return

    def t_ctrl_end_ctrl(self,t):
        r'%\}'
        t.lexer.begin('INITIAL')
        return

    t_ctrl_CTRL = r'[^%]+'


    t_TEXT = r"[^\{]+"

    #def t_newline(t):
    #    r'\n'
    #    t.lexer.lineno += 1

    def t_error(self,t):
        pass

    def t_var_error(self,t):
        pass

    def t_ctrl_error(self,t):
        pass

    def __init__(self, **kwargs):
        self.lexer = lex.lex(module=self, **kwargs)
        return
    
    def i18n(self, s):
        self.lexer.input(s)
        result=""
        while True:
            tok = self.lexer.token()
            if tok is None: break
            if tok.type == 'TEXT':
                string = s[tok.lexpos:tok.lexpos+len(tok.value)]
                result += markup(string)
            elif tok.type == "VAR":
                result += "{{" + tok.value + "}}"
            elif tok.type == "CTRL":
                result += "{%" + tok.value + "%}"
        return result

TRANSLEXER = TransLexer()

def print_text(s):
    if isinstance(s, bs4.element.NavigableString):
        s=str(s)
        if s.strip():
            print(s, end="")
    return ""

def prepare_translation(elt):
    if isinstance(elt, bs4.element.NavigableString):
        # on a un élément de type texte
        # on va sauter toutes les parties de plus d'un séparateur contigu
        # et tous le marquages destinée à Jinja2, c'est à dire les
        # parenthèses de type {{...}} et {%...%}
        if str(elt).strip() and not isinstance(elt, bs4.Comment):
            return TRANSLEXER.i18n(str(elt))
    return ""
            
def parcours_de_textes(doc, a_faire):
    """
    Parcourt un document XML, et pour chaque textes qu'on y trouve,
    lance la fonction a_faire
    @param doc un arbre xml
    @param a_faire un fonction de profil str -> str ; la chaîne de
           sortie peut être obtenue à partir de la chaîne en entrée, et
           si sa valeur n'est pas "", elle est utilisable pour remplacer
           la chaîne d'entrée
    @ return un nouveau document XML
    """

    # N.B. avec un élément e de type Tag, quand on veut remplacer son texte,
    # if faut le faire avec e.string.replace_with(nouvelle_valeur)
    for elt in doc.children:
        t = a_faire(elt)
        if t:
            elt.string.replace_with(t)
        if isinstance(elt, bs4.element.Tag) and elt.name.lower() != 'script':
            parcours_de_textes(elt, a_faire)
    return

if __name__ == "__main__":
    xmlstring = open(sys.argv[1]).read()
    restoreDocType = False
    if "<!DOCTYPE html>" in xmlstring:
        ############ protection du doctype #################
        xmlstring = xmlstring.replace("<!DOCTYPE html>", "{{ !DOCTYPE html }}")
        restoreDocType = True
    doc = BS(xmlstring, 'html.parser')
    parcours_de_textes(doc, prepare_translation)
    newxmlstring = doc.prettify()
    if restoreDocType:
        newxmlstring = newxmlstring.replace(
            "{{ !DOCTYPE html }}", "<!DOCTYPE html>")
    ################ marquage des attributs title #############
    pattern = re.compile(r"title=\"([^\"]*+)\"")
    newxmlstring = pattern.sub("title=\"{% translate '\\1' %}\"", newxmlstring)
    print(newxmlstring)
    



# fonctions communes et constantes communes

import re

def safeText(text):
    result=text
    desaccent = {
        r"[àâä]" : "a",
        r"[ÀÂÄ]" : "A",
        r"[éèêë]" : "e",
        r"[ÉÈÊË]" : "E",
        r"[îï]" : "i",
        r"[ÎÏ]" : "I",
        r"[ôö]" : "o",
        r"[ÔÖ]" : "O",
        r"[ùûü]" : "u",
        r"[ÙÛÜ]" : "U",
        r"[ç]" : "c",
        r"[Ç]" : "C",
    }
    for r, repl in desaccent.items():
        result = re.sub(r,repl, result)
    return result
    
def safeIdent(text):
    return re.sub(r"""[. ,;!"'?]""", "_", safeText(text).replace("°", ""))



"""
Importation de données de Gemasco vers SLM,
sans modifier la structure de la base de données
"""

from PyQt6.QtWidgets import QApplication, QMainWindow
from PyQt6.QtGui import QFileSystemModel
from PyQt6.QtCore import QDir, QProcess, QThread

from Ui_importeDeGemasco import Ui_MainWindow

import sys
import os

class MyMW (QMainWindow, Ui_MainWindow):
    def __init__(self, parent = None, argv=[]):
        QMainWindow.__init__(self, parent)
        self.argv = argv
        self.setupUi(self)
        self.actionChoix_du_r_pertoire.triggered.connect(self.choix_repertoire)
        self.actionXslx_CSV.triggered.connect(self.runConversions)
        self.convertButton.clicked.connect(self.runConversions)
        self.actionQuitter.triggered.connect(self.close)
        self.pwd = os.getcwd()
        if len(argv) >= 2:
            self.pwd = argv[1]
        self.xls_view = None
        self.csv_view = None
        # mise en place de l'arbre des répertoires
        self.dirModel = QFileSystemModel()
        dm = self.dirModel
        dm.setRootPath(QDir.rootPath())
        dm.setFilter(QDir.Filter.NoDotAndDotDot | QDir.Filter.AllDirs)
        self.dir_treeView.setModel(dm)
        self.dir_treeView.setRootIndex(dm.index(self.pwd))
        self.dir_treeView.clicked.connect(self.on_dir_clicked)
        self.dir_treeView.setColumnWidth(0, 400)
        self.initExcelCsvListViews()
        # place d'abord l'onglet des répertoires
        self.tabWidget.setCurrentIndex(0)
        self.idealThreadCount = QThread.idealThreadCount()
        return

    def on_dir_clicked(self, index):
        path = self.dirModel.fileInfo(index).absoluteFilePath()
        self.excel_listView.setRootIndex(self.excel_fileModel.setRootPath(path))
        self.csv_listView.setRootIndex(self.csv_fileModel.setRootPath(path))
        self.pwd = path
        return

    def initExcelCsvListViews(self):
        """
        Affiche les contenus de self.excel_listView et self.csv_listView
        en se basant sur self.pwd
        """
        def filemodel(names):
            fm = QFileSystemModel()
            fm.setFilter(QDir.Filter.NoDotAndDotDot |  QDir.Filter.Files)
            fm.setNameFilters(names)
            fm.setNameFilterDisables(False) # don't show when filtered out
            return fm
        
        fm = filemodel(["*.xlsx"])
        self.excel_listView.setModel(fm)
        self.excel_listView.setRootIndex(fm.setRootPath(self.pwd))
        self.excel_fileModel = fm
       
        fm = filemodel(["*.csv"])
        self.csv_listView.setModel(fm)
        self.csv_listView.setRootIndex(fm.setRootPath(self.pwd))
        self.csv_fileModel = fm
        return
    
    def choix_repertoire(self):
        self.tabWidget.setCurrentIndex(0)
        return

    def tabActions(self):
        self.tabWidget.setCurrentIndex(1)
        return

    convertIndex = 0      # sera incrémenté par newConvertProcess
    listeConversions = {} # en fait, dictionnaire int => instance de QProcess
    
   
    @staticmethod
    def newConvertProcess(infile, out, err, callback):
        """
        Crée un nouveau processus de conversion, prêt à démarrer avec start(),
        et lui associe un numéro autoincrémenté
        @param infile le fichier XSLX utilisé en entrée
        @param out une instance de QPlainTextEdit
        @param err une instance de QPlainTextEdit
        @param callback une fonction de rappel pour traiter les changements
               d'état du processus et sa terminaison ; le profil de cette
               fonction est (index:int, message) => None
        @return None ; mais modifie par effet de bord MyMW.convertIndex et
                MyMW.listeConversions
        """
        p = QProcess()
        MyMW.convertIndex += 1
        index = MyMW.convertIndex
        prefix = f"Conversion n° {index} : "
        
        def report_stdout():
            data = p.readAllStandardOutput()
            out.appendPlainText(prefix + bytes(data).decode("utf8"))
            return
        
        def report_stderr():
            data = p.readAllStandardError()
            err.appendPlainText(prefix + bytes(data).decode("utf8"))
            return

        def report_state(s):
            callback(index, s)
            return

        def cleanup():
            callback(index, "end")
            return
        
        p.readyReadStandardOutput.connect(report_stdout)
        p.readyReadStandardError.connect(report_stderr)
        p.stateChanged.connect(report_state)
        p.finished.connect(cleanup)
        p.setProgram("/usr/bin/libreoffice")
        p.setArguments(["--headless",
                        "--convert-to",
                        "csv",
                        infile,
                        "--outdir",
                        os.path.dirname(infile),
                        ])
        MyMW.listeConversions[index] = {"process": p, "fired": False}
        return

    conversions_actives = 0
    
    def activeConversionsDormantes(self):
        if self.conversions_actives >= self.idealThreadCount:
            return
        dormants = [p for i,p in self.listeConversions.items() \
                    if not p["fired"]]
        # on lance autant de programmes qu'il est bon d'en lancer
        for i in range (min(len(dormants),
                            self.idealThreadCount - self.conversions_actives)):
            dormants[i]["fired"] = True
            dormants[i]["process"].start()
            self.conversions_actives +=1
            print("GRRRRR on lance un processus dormant :", dormants[i])
        return
        
    def runConversions(self):
        deja_convertis = []
        model = self.csv_listView.model()
        idx = model.index(model.rootPath())
        for i in range(0, model.rowCount(idx)):
            mi = model.index(i, 0, idx)
            deja_convertis.append(model.fileInfo(mi).absoluteFilePath())
        
        a_convertir = []
        model = self.excel_listView.model()
        idx = model.index(model.rootPath())
        for i in range(0, model.rowCount(idx)):
            mi = model.index(i, 0, idx)
            path = model.fileInfo(mi).absoluteFilePath()
            if path.replace(".xlsx",".csv") in deja_convertis:
                continue
            a_convertir.append(path)
        print("GRRRR deja_convertis", deja_convertis)
        print("GRRRR a_convertir", a_convertir)

        self.tabActions()
        def cb(index, message):
            """
            fonction de rappel pour les changements d'état des process et leur
            terminaison
            """
            print("rappel GRRRR process", index, message)
            if message == "end":
                # terminaison du programme d'index 'index'.
                print(f"GRRRR le process {index} est fini")
                self.conversions_actives -= 1
                # on vient de finir un process,
                # voyons s'il en reste d'autres à activer
                self.activeConversionsDormantes()
            return
        
        for infile in a_convertir:
            self.newConvertProcess(
                infile, self.stdout_TextEdit, self.stderr_TextEdit, cb)
        self.activeConversionsDormantes()
        return

    def cleanup(self):
        print("GRRRR: Process finished.")
        self.process = None
        return

    def report_stdout(self):
        data = self.process.readAllStandardOutput()
        stdout = bytes(data).decode("utf8")
        print("GRRRR stdout :", stdout)
        self.stdout_TextEdit.appendPlainText(stdout)
        return

    def report_stderr(self):
        data = self.process.readAllStandardError()
        stderr = bytes(data).decode("utf8")
        print("GRRRR stderr :", stderr)
        self.stderr_TextEdit.appendPlainText(stderr)
        return

    def report_state(self, s):
        print("GRRRR state :", s)
        return

    
class MyApp(QApplication):
    def __init__(self, argv):
        QApplication.__init__(self, argv)
        global mw
        mw = MyMW(argv=argv)
        mw.show()
        return

# objet temporaire ; contiendra la fenêtre principale créée par effet de bord
mw = None

# crée mw en tenant compte des arguments
app = MyApp(sys.argv)
app.exec()



from django.forms import ModelForm, Form, PasswordInput, \
   HiddenInput, ValidationError
from django import forms
from django.utils.translation import gettext_lazy as _

from .models import Parametres, Courriers, Disciplines, Materiel, \
   Classes, Eleves

import re

class donnees_utilisateur_Form(Form):
   labelNom = _("Nom")
   nom = forms.CharField(label=labelNom, max_length=25, required = False)
   prenom = forms.CharField(label=_("Prénom"), max_length=25, required = False)
   email = forms.CharField(label=_("Courriel"), max_length=50, required = False)
   passe1 = forms.CharField(label=_("Mot de passe"), max_length=32,
                            widget=PasswordInput, required = False)
   passe2 = forms.CharField(label=_("Répétez le mot de passe"), max_length=32,
                            widget=PasswordInput, required = False)
   next = forms.CharField(max_length=50, widget = HiddenInput)

   def __init__(self, *args, **kwargs):
      super(donnees_utilisateur_Form, self).__init__(*args, **kwargs)
      self.fields['nom'].widget.attrs['placeholder'] = 'Votre nom ?'
      self.fields['prenom'].widget.attrs['placeholder'] = 'Votre prénom ?'
      self.fields['email'].widget.attrs['placeholder'] = 'Votre courriel ?'
      self.fields['passe1'].widget.attrs['placeholder'] = \
         'Nouveau mot de passe éventuel'
      self.fields['passe2'].widget.attrs['placeholder'] = 'Le même ...'
      return
     
   def clean_passe2(self):
      p1 = self.cleaned_data["passe1"]
      p2 = self.cleaned_data["passe2"]
      if p1 and not p2 == p1:
         raise ValidationError(
            _("Erreur : les deux mots de passes étaient différents, recommencez"))
      if p1 and len(p1) < 7:
         raise ValidationError(
            _("Erreur : un mot de passe doit contenir au moins 7 caractères"))
      minuscules = re.findall(r"[a-z]", p1)
      majuscules = re.findall(r"[A-Z]", p1)
      chiffres = re.findall(r"\d", p1)
      if p1 and (not minuscules or not majuscules or not chiffres):
          raise ValidationError(
            _("Erreur : un mot de passe doit contenir au moins ") + \
             _("une lettre minuscule, une majuscule, et un chiffre"))
      return p2


class Parametres_Form(ModelForm):
   class Meta:
        model = Parametres
        fields = '__all__'
        
class Courriers_Form(ModelForm):
   class Meta:
        model = Courriers
        fields = '__all__'
        exclude = ('N',)

class Materiel_Form(ModelForm):
   discipline = forms.ModelChoiceField(queryset=Disciplines.objects.order_by('libelle'))
   class Meta:
      model = Materiel
      fields = '__all__'
      exclude = ('id','Pretes','Achetes','Perdus')
  
class Eleves_Form(ModelForm):
   classe = forms.ModelChoiceField(queryset=Classes.classes_importantes())
   class Meta:
      model = Eleves
      fields = ('Nom_de_famille', 'Prenom', 'classe',
                'classes_passees', 'no_caution',
                'Lib_Mat_Enseignee_1','Lib_Mat_Enseignee_2',
                'Lib_Mat_Enseignee_3','Lib_Mat_Enseignee_4',
                'Lib_Mat_Enseignee_5','Lib_Mat_Enseignee_6',
                'Lib_Mat_Enseignee_7','Lib_Mat_Enseignee_8',
                'Lib_Mat_Enseignee_9','Lib_Mat_Enseignee_10',
                'Tel_Portable', 'Adresse_Repr_Leg',
                'Tel_Portable_Repr_Leg', 'Tel_Portable_Autre_Repr_Leg',
                'a_verifier')
      labels = {f.name: _(f.verbose_name) for f in Eleves._meta.fields}
      widgets = {
         'classes_passees': forms.TextInput(attrs={'readonly': 'readonly'}),
         'no_caution': forms.TextInput(attrs={'readonly': 'readonly'}),
         'a_verifier': forms.TextInput(attrs={'readonly': 'readonly'}),
      }

class Eleves_ancien_Form(ModelForm):
   """
   Formulaire pour les élèves dont le numéro de caution n'avait pas été
   attribué automatiquement par le logiciel, c'est à dire avec un numéro de
   caution inférieur à MIN_NO_CAUTION
   """
   classe = forms.ModelChoiceField(queryset=Classes.classes_importantes())
   class Meta:
      model = Eleves
      fields = ('Nom_de_famille', 'Prenom', 'classe',
                'classes_passees', 'no_caution',
                'Lib_Mat_Enseignee_1','Lib_Mat_Enseignee_2',
                'Lib_Mat_Enseignee_3','Lib_Mat_Enseignee_4',
                'Lib_Mat_Enseignee_5','Lib_Mat_Enseignee_6',
                'Lib_Mat_Enseignee_7','Lib_Mat_Enseignee_8',
                'Lib_Mat_Enseignee_9','Lib_Mat_Enseignee_10',
                'Tel_Portable', 'Adresse_Repr_Leg',
                'Tel_Portable_Repr_Leg', 'Tel_Portable_Autre_Repr_Leg',
                'a_verifier')
      labels = {f.name: _(f.verbose_name) for f in Eleves._meta.fields}
      widgets = {
         'classes_passees': forms.TextInput(attrs={'readonly': 'readonly'}),
         'a_verifier': forms.TextInput(attrs={'readonly': 'readonly'}),
      }

class SiecleForm(forms.Form):
    fichier_siecle = forms.FileField(label='Sélectionner un export de SIÈCLE')



"""i18n_subsites plugin creates i18n-ized subsites of the default site

This plugin is designed for Pelican 3.4 and later
"""


import os
import six
import logging
import posixpath

from copy import copy
from itertools import chain
from operator import attrgetter
try:
    from collections.abc import OrderedDict
except ImportError:
    from collections import OrderedDict
from contextlib import contextmanager
from six.moves.urllib.parse import urlparse

import gettext
import locale

from pelican import signals
from pelican.generators import ArticlesGenerator, PagesGenerator
from pelican.settings import configure_settings
try:
    from pelican.contents import Draft
except ImportError:
    from pelican.contents import Article as Draft


# Global vars
_MAIN_SETTINGS = None     # settings dict of the main Pelican instance
_MAIN_LANG = None         # lang of the main Pelican instance
_MAIN_SITEURL = None      # siteurl of the main Pelican instance
_MAIN_STATIC_FILES = None # list of Static instances the main Pelican instance
_SUBSITE_QUEUE = {}   # map: lang -> settings overrides
_SITE_DB = OrderedDict()           # OrderedDict: lang -> siteurl
_SITES_RELPATH_DB = {}       # map: (lang, base_lang) -> relpath
# map: generator -> list of removed contents that need interlinking
_GENERATOR_DB = {}
_NATIVE_CONTENT_URL_DB = {} # map: source_path -> content in its native lang
_LOGGER = logging.getLogger(__name__)


@contextmanager
def temporary_locale(temp_locale=None):
    '''Enable code to run in a context with a temporary locale

    Resets the locale back when exiting context.
    Can set a temporary locale if provided
    '''
    orig_locale = locale.setlocale(locale.LC_ALL)
    if temp_locale is not None:
        locale.setlocale(locale.LC_ALL, temp_locale)
    yield
    locale.setlocale(locale.LC_ALL, orig_locale)


def initialize_dbs(settings):
    '''Initialize internal DBs using the Pelican settings dict

    This clears the DBs for e.g. autoreload mode to work
    '''
    global _MAIN_SETTINGS, _MAIN_SITEURL, _MAIN_LANG, _SUBSITE_QUEUE
    _MAIN_SETTINGS = settings
    _MAIN_LANG = settings['DEFAULT_LANG']
    _MAIN_SITEURL = settings['SITEURL']
    _SUBSITE_QUEUE = settings.get('I18N_SUBSITES', {}).copy()
    prepare_site_db_and_overrides()
    # clear databases in case of autoreload mode
    _SITES_RELPATH_DB.clear()
    _NATIVE_CONTENT_URL_DB.clear()
    _GENERATOR_DB.clear()


def prepare_site_db_and_overrides():
    '''Prepare overrides and create _SITE_DB

    _SITE_DB.keys() need to be ready for filter_translations
    '''
    _SITE_DB.clear()
    _SITE_DB[_MAIN_LANG] = _MAIN_SITEURL
    # make sure it works for both root-relative and absolute
    main_siteurl = '/' if _MAIN_SITEURL == '' else _MAIN_SITEURL
    for lang, overrides in _SUBSITE_QUEUE.items():
        if 'SITEURL' not in overrides:
            overrides['SITEURL'] = posixpath.join(main_siteurl, lang)
        _SITE_DB[lang] = overrides['SITEURL']
        # default subsite hierarchy
        if 'OUTPUT_PATH' not in overrides:
            overrides['OUTPUT_PATH'] = os.path.join(
                _MAIN_SETTINGS['OUTPUT_PATH'], lang)
        if 'CACHE_PATH' not in overrides:
            overrides['CACHE_PATH'] = os.path.join(
                _MAIN_SETTINGS['CACHE_PATH'], lang)
        if 'STATIC_PATHS' not in overrides:
            overrides['STATIC_PATHS'] = []
        if ('THEME' not in overrides and 'THEME_STATIC_DIR' not in overrides and
                'THEME_STATIC_PATHS' not in overrides):
            relpath = relpath_to_site(lang, _MAIN_LANG)
            overrides['THEME_STATIC_DIR'] = posixpath.join(
                relpath, _MAIN_SETTINGS['THEME_STATIC_DIR'])
            overrides['THEME_STATIC_PATHS'] = []
        # to change what is perceived as translations
        overrides['DEFAULT_LANG'] = lang


def subscribe_filter_to_signals(settings):
    '''Subscribe content filter to requested signals'''
    for sig in settings.get('I18N_FILTER_SIGNALS', []):
        sig.connect(filter_contents_translations)


def initialize_plugin(pelican_obj):
    '''Initialize plugin variables and Pelican settings'''
    if _MAIN_SETTINGS is None:
        initialize_dbs(pelican_obj.settings)
        subscribe_filter_to_signals(pelican_obj.settings)


def get_site_path(url):
    '''Get the path component of an url, excludes siteurl

    also normalizes '' to '/' for relpath to work,
    otherwise it could be interpreted as a relative filesystem path
    '''
    path = urlparse(url).path
    if path == '':
        path = '/'
    return path


def relpath_to_site(lang, target_lang):
    '''Get relative path from siteurl of lang to siteurl of base_lang

    the output is cached in _SITES_RELPATH_DB
    '''
    path = _SITES_RELPATH_DB.get((lang, target_lang), None)
    if path is None:
        siteurl = _SITE_DB.get(lang, _MAIN_SITEURL)
        target_siteurl = _SITE_DB.get(target_lang, _MAIN_SITEURL)
        path = posixpath.relpath(get_site_path(target_siteurl),
                                 get_site_path(siteurl))
        _SITES_RELPATH_DB[(lang, target_lang)] = path
    return path


def save_generator(generator):
    '''Save the generator for later use

    initialize the removed content list
    '''
    _GENERATOR_DB[generator] = []


def article2draft(article):
    '''Transform an Article to Draft'''
    draft = Draft(article._content, article.metadata, article.settings,
                  article.source_path, article._context)
    draft.status = 'draft'
    return draft


def page2hidden_page(page):
    '''Transform a Page to a hidden Page'''
    page.status = 'hidden'
    return page


class GeneratorInspector(object):
    '''Inspector of generator instances'''

    generators_info = {
        ArticlesGenerator: {
            'translations_lists': ['translations', 'drafts_translations'],
            'contents_lists': [('articles', 'drafts')],
            'hiding_func': article2draft,
            'policy': 'I18N_UNTRANSLATED_ARTICLES',
        },
        PagesGenerator: {
            'translations_lists': ['translations', 'hidden_translations'],
            'contents_lists': [('pages', 'hidden_pages')],
            'hiding_func': page2hidden_page,
            'policy': 'I18N_UNTRANSLATED_PAGES',
        },
    }

    def __init__(self, generator):
        '''Identify the best known class of the generator instance

        The class '''
        self.generator = generator
        self.generators_info.update(generator.settings.get(
            'I18N_GENERATORS_INFO', {}))
        for cls in generator.__class__.__mro__:
            if cls in self.generators_info:
                self.info = self.generators_info[cls]
                break
        else:
            self.info = {}

    def translations_lists(self):
        '''Iterator over lists of content translations'''
        return (getattr(self.generator, name) for name in
                self.info.get('translations_lists', []))

    def contents_list_pairs(self):
        '''Iterator over pairs of normal and hidden contents'''
        return (tuple(getattr(self.generator, name) for name in names)
                for names in self.info.get('contents_lists', []))

    def hiding_function(self):
        '''Function for transforming content to a hidden version'''
        hiding_func = self.info.get('hiding_func', lambda x: x)
        return hiding_func

    def untranslated_policy(self, default):
        '''Get the policy for untranslated content'''
        return self.generator.settings.get(self.info.get('policy', None),
                                           default)

    def all_contents(self):
        '''Iterator over all contents'''
        translations_iterator = chain(*self.translations_lists())
        return chain(translations_iterator,
                     *(pair[i] for pair in self.contents_list_pairs()
                       for i in (0, 1)))


def filter_contents_translations(generator):
    '''Filter the content and translations lists of a generator

    Filters out
        1) translations which will be generated in a different site
        2) content that is not in the language of the currently
        generated site but in that of a different site, content in a
        language which has no site is generated always. The filtering
        method bay be modified by the respective untranslated policy
    '''
    inspector = GeneratorInspector(generator)
    current_lang = generator.settings['DEFAULT_LANG']
    langs_with_sites = _SITE_DB.keys()
    removed_contents = _GENERATOR_DB[generator]

    for translations in inspector.translations_lists():
        for translation in translations[:]:    # copy to be able to remove
            if translation.lang in langs_with_sites:
                translations.remove(translation)
                removed_contents.append(translation)

    hiding_func = inspector.hiding_function()
    untrans_policy = inspector.untranslated_policy(default='hide')
    for (contents, other_contents) in inspector.contents_list_pairs():
        for content in other_contents: # save any hidden native content first
            if content.lang == current_lang: # in native lang
                # save the native URL attr formatted in the current locale
                _NATIVE_CONTENT_URL_DB[content.source_path] = content.url
        for content in contents[:]:        # copy for removing in loop
            if content.lang == current_lang: # in native lang
                # save the native URL attr formatted in the current locale
                _NATIVE_CONTENT_URL_DB[content.source_path] = content.url
            elif content.lang in langs_with_sites and untrans_policy != 'keep':
                contents.remove(content)
                if untrans_policy == 'hide':
                    other_contents.append(hiding_func(content))
                elif untrans_policy == 'remove':
                    removed_contents.append(content)
    return


def install_templates_translations(generator):
    '''Install gettext translations in the jinja2.Environment

    Only if the 'jinja2.ext.i18n' jinja2 extension is enabled
    the translations for the current DEFAULT_LANG are installed.
    '''
    if 'JINJA_ENVIRONMENT' in generator.settings: # pelican 3.7+
        jinja_extensions = generator.settings['JINJA_ENVIRONMENT'].get(
            'extensions', [])
    else:
        jinja_extensions = generator.settings['JINJA_EXTENSIONS']

    if 'jinja2.ext.i18n' in jinja_extensions:
        domain = generator.settings.get('I18N_GETTEXT_DOMAIN', 'messages')
        localedir = generator.settings.get('I18N_GETTEXT_LOCALEDIR')
        if localedir is None:
            localedir = os.path.join(generator.theme, 'translations')
        current_lang = generator.settings['DEFAULT_LANG']
        if current_lang == generator.settings.get('I18N_TEMPLATES_LANG',
                                                  _MAIN_LANG):
            translations = gettext.NullTranslations()
        else:
            langs = [current_lang]
            try:
                translations = gettext.translation(domain, localedir, langs)
            except (IOError, OSError):
                _LOGGER.error((
                    "Cannot find translations for language '{}' in '{}' with "
                    "domain '{}'. Installing NullTranslations.").format(
                        langs[0], localedir, domain))
                translations = gettext.NullTranslations()
        newstyle = generator.settings.get('I18N_GETTEXT_NEWSTYLE', True)
        generator.env.install_gettext_translations(translations, newstyle)


def add_variables_to_context(generator):
    '''Adds useful iterable variables to template context'''
    context = generator.context             # minimize attr lookup
    context['relpath_to_site'] = relpath_to_site
    context['main_siteurl'] = _MAIN_SITEURL
    context['main_lang'] = _MAIN_LANG
    context['lang_siteurls'] = _SITE_DB
    current_lang = generator.settings['DEFAULT_LANG']
    extra_siteurls = _SITE_DB.copy()
    extra_siteurls.pop(current_lang)
    context['extra_siteurls'] = extra_siteurls


def interlink_translations(content):
    '''Link content to translations in their main language

    so the URL (including localized month names) of the different subsites
    will be honored
    '''
    lang = content.lang
    # sort translations by lang
    content.translations.sort(key=attrgetter('lang'))
    for translation in content.translations:
        relpath = relpath_to_site(lang, translation.lang)
        if translation.source_path in _NATIVE_CONTENT_URL_DB:
            url = _NATIVE_CONTENT_URL_DB[translation.source_path]
            translation.override_url = posixpath.join(relpath, url)
        else:
            # do not override the URL when "translate: true" is in the metadata
            pass
    return


def interlink_translated_content(generator):
    '''Make translations link to the native locations

    for generators that may contain translated content
    '''
    inspector = GeneratorInspector(generator)
    for content in inspector.all_contents():
        interlink_translations(content)


def interlink_removed_content(generator):
    '''For all contents removed from generation queue update interlinks

    link to the native location
    '''
    current_lang = generator.settings['DEFAULT_LANG']
    for content in _GENERATOR_DB[generator]:
        if content.source_path in _NATIVE_CONTENT_URL_DB:
            url = _NATIVE_CONTENT_URL_DB[content.source_path]
            relpath = relpath_to_site(current_lang, content.lang)
            content.override_url = posixpath.join(relpath, url)
        else:
            # do not override the URL when "translate: true" is in the metadata
            pass
    return


def interlink_static_files(generator):
    '''Add links to static files in the main site if necessary'''
    if generator.settings['STATIC_PATHS'] != []:
        return                               # customized STATIC_PATHS
    try: # minimize attr lookup
        static_content = generator.context['static_content']
    except KeyError:
        static_content = generator.context['filenames']
    relpath = relpath_to_site(generator.settings['DEFAULT_LANG'], _MAIN_LANG)
    for staticfile in _MAIN_STATIC_FILES:
        if staticfile.get_relative_source_path() not in static_content:
            staticfile = copy(staticfile) # prevent override in main site
            staticfile.override_url = posixpath.join(relpath, staticfile.url)
            try:
                generator.add_source_path(staticfile, static=True)
            except TypeError:
                generator.add_source_path(staticfile)


def save_main_static_files(static_generator):
    '''Save the static files generated for the main site'''
    global _MAIN_STATIC_FILES
    # test just for current lang as settings change in autoreload mode
    if static_generator.settings['DEFAULT_LANG'] == _MAIN_LANG:
        _MAIN_STATIC_FILES = static_generator.staticfiles


def update_generators():
    '''Update the context of all generators

    Ads useful variables and translations into the template context
    and interlink translations
    '''
    for generator in _GENERATOR_DB.keys():
        install_templates_translations(generator)
        add_variables_to_context(generator)
        interlink_static_files(generator)
        interlink_removed_content(generator)
        interlink_translated_content(generator)


def get_pelican_cls(settings):
    '''Get the Pelican class requested in settings'''
    cls = settings['PELICAN_CLASS']
    if isinstance(cls, six.string_types):
        module, cls_name = cls.rsplit('.', 1)
        module = __import__(module)
        cls = getattr(module, cls_name)
    return cls


def create_next_subsite(pelican_obj):
    '''Create the next subsite using the lang-specific config

    If there are no more subsites in the generation queue, update all
    the generators (interlink translations and removed content, add
    variables and translations to template context). Otherwise get the
    language and overrides for next the subsite in the queue and apply
    overrides.  Then generate the subsite using a PELICAN_CLASS
    instance and its run method. Finally, restore the previous locale.
    '''
    global _MAIN_SETTINGS
    if len(_SUBSITE_QUEUE) == 0:
        _LOGGER.debug(
            'i18n: Updating cross-site links and context of all generators.')
        update_generators()
        _MAIN_SETTINGS = None             # to initialize next time
    else:
        with temporary_locale():
            settings = _MAIN_SETTINGS.copy()
            lang, overrides = _SUBSITE_QUEUE.popitem()
            settings.update(overrides)
            settings = configure_settings(settings)      # to set LOCALE, etc.
            cls = get_pelican_cls(settings)

            new_pelican_obj = cls(settings)
            _LOGGER.debug(("Generating i18n subsite for language '{}' "
                           "using class {}").format(lang, cls))
            new_pelican_obj.run()


# map: signal name -> function name
_SIGNAL_HANDLERS_DB = {
    'get_generators': initialize_plugin,
    'article_generator_pretaxonomy': filter_contents_translations,
    'page_generator_finalized': filter_contents_translations,
    'get_writer': create_next_subsite,
    'static_generator_finalized': save_main_static_files,
    'generator_init': save_generator,
}


def register():
    '''Register the plugin only if required signals are available'''
    for sig_name in _SIGNAL_HANDLERS_DB.keys():
        if not hasattr(signals, sig_name):
            _LOGGER.error((
                'The i18n_subsites plugin requires the {} '
                'signal available for sure in Pelican 3.4.0 and later, '
                'plugin will not be used.').format(sig_name))
            return

    for sig_name, handler in _SIGNAL_HANDLERS_DB.items():
        sig = getattr(signals, sig_name)
        sig.connect(handler)



"""
    gestion.version, un module pour SLM
    - Ce petit module renvoie juste le numéro de version, éventuelement
      calculé en examinant des git-tags

    Copyright (C) 2023-2024 Georges Khaznadar <georgesk@debian.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from subprocess import run
from packaging.version import Version

############ modifications must be done here ##############################
major = 1
minor = 3
flavor = ".0"
############ end of modifiable section      ###############################

this_version = f"{major}.{minor}"
if flavor:
    this_version = f"{major}.{minor}{flavor}"


p = run("git tag | grep '^v'| sed 's/^v//'", shell=True, capture_output=True)

versions = [v for v in p.stdout.strip().decode("utf-8").split("\n") if v]
versions = sorted(versions, key = Version)

def create_new_tag(verbose=True):
    """
    Create a vew tag "v"+this_version when it does not exist already
    """
    if (not versions) or Version(this_version) > Version(versions[-1]):
        run(f"git tag v{this_version}", shell=True)
        if verbose:
            print(f"added the tag v{this_version}")

if __name__ == "__main__":
    if (not versions) or Version(this_version) > Version(versions[-1]):
        print(this_version)
        print("already tagged:", " ".join(versions))
    else:
        print(f"The current version is {this_version}, and is in the already tagged versions: {versions}")
    



# This file is only used if you use `make publish` or
# explicitly specify it as your config file.

from .pelicanconf import *

# If your site is available via HTTPS, make sure SITEURL begins with https://
SITEURL = 'https://manuels.freeduc.science'
RELATIVE_URLS = True

FEED_ALL_ATOM = 'feeds/all.atom.xml'
CATEGORY_FEED_ATOM = 'feeds/{slug}.atom.xml'

DELETE_OUTPUT_DIRECTORY = True

# Following items are often useful when publishing

# DISQUS_SITENAME = ""
# GOOGLE_ANALYTICS = ""



from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.contrib.auth.models import User

from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service as ChromeService
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys

from .models import Eleves, Inventaire, Materiel

import os

# Create your tests here.

class TestAvecBDD(StaticLiveServerTestCase):
    """
    Tests qui nécessitent de déployer une base de données de tests
    """

    fixtures = ['fixtures/test_data.json']
    
    credentials = {
        'username': 'testuser',
        'password': 'secret'
    }
    
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.service = ChromeService(executable_path="/usr/bin/chromedriver")
        options = Options()
        cls.ui_visible = True
        if not os.environ.get("VIEW_SELENIUM") == "1":
            options.add_argument('--headless')
            print("WARNING: to display Chrome's UI, set VIEW_SELENIUM=1")
            cls.ui_visible = False
        cls.selenium = webdriver.Chrome(
            options = options, service = cls.service)
        cls.selenium.implicitly_wait(10)
        return

    def setUp(self):
        """
        Ajoute un utilisateur 'testuser'
        """
        StaticLiveServerTestCase.setUp(self)
        u = User.objects.create_user(**self.credentials)
        u.is_staff = True # important, car le login est à /admin/login
        u.save()
        return
        
    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super().tearDownClass()
        return
    
    def test_inventaire(self):
        # nombre total de livres
        livres = Inventaire.objects.all()
        self.assertGreater(
            len(livres), 3000, "on attendrait plus de 3000 livres")
        # dont « non prêtés »
        self.assertEqual(
            Inventaire.non_pretes_qs().count(),
            1983, "on attendrait 1983 livres non prêtés")
        # dont « prêtés »
        self.assertEqual(
            Inventaire.pretes_qs().count(),
            1933, "on attendrait 1933 livres prêtés")
        # manuels "Physique Chimie 2nde" prêtés
        ref = Materiel.objects.get(titre="Physique Chimie 2nde")
        qs = Inventaire.pretes_qs()
        qs = qs.filter(materiel = ref)
        self.assertEqual(
            qs.count(), 59,
            "On attendrait 59 livres de Physique Chimie 2nde")
        # À qui est ce livre ?
        liv = Inventaire.objects.get(id = 227)
        self.assertEqual(
            liv.materiel.titre, "Team deutsch 2nde",
            "le titre du livre n°227 devrait être Team deutsch 2nde")
        e = liv.actuellement_a()
        self.assertIs(
            e, None, "le livre Team deutsch 2nde n° 227 ne devrait pas prêté")
        liv = Inventaire.objects.get(id = 667)
        self.assertEqual(
            liv.materiel.titre, "Gammar training time 2nde",
            "le titre du livre n°667 devrait être Gammar training time 2nde")
        e = liv.actuellement_a()
        f = Eleves.objects.get(NomPrenomEleve = "Bifidus Celestine")
        self.assertEqual(
            e.id, f.id,
            "le livre Gammar training time 2nde n° 667 devrait être prêté à " +
            "Bifidus Celestine")
        return
    
    def test_avec_login(self):
        """
        - force l'interface en français et vérifie les traductions
        - force l'interface en espagnol et vérifie les traductions
        - login sous l'identité de 'testuser'
        """

        """
        I18N et L10N
        ============
        
        Sur la page '/apropos', qui ne requiert pas d'être logé, vérifions
        si un changement de langue va modifier par exemple le titre d'un
        bouton.
        """
        self.selenium.get(f"{self.live_server_url}/apropos")
        # choix du français
        self.selenium.find_element(
            By.CSS_SELECTOR, "#language-fr img").click()
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        img = self.selenium.find_element(By.CSS_SELECTOR, '#aide img')
        self.assertEqual(img.get_attribute("title"), "Aide contextuelle")
        # choix de l'espagnol
        img_es = self.selenium.find_element(
            By.CSS_SELECTOR, "#language-es img").click()
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        img = self.selenium.find_element(By.CSS_SELECTOR, '#aide img')
        self.assertEqual(img.get_attribute("title"), "Ayuda contextual")

        """
        Login en tant que 'testuser'
        """
        self.selenium.find_element(By.CSS_SELECTOR, '#id_username')\
                     .send_keys(self.credentials['username'])
        self.selenium.find_element(By.CSS_SELECTOR, '#id_password')\
                     .send_keys(self.credentials['password'])
        form = self.selenium.find_element(By.TAG_NAME, "form")
        valid = form.find_elements(By.TAG_NAME, "input")[-1]
        valid.click();
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()

        """
        Fonctionnement de l'auto-complétion de nom d'utilisateur dans la
        page '/prets'
        """
        self.selenium.get(f"{self.live_server_url}/prets")
        nomprenom = self.selenium.find_element(By.ID, "nomprenom")
        ActionChains(self.selenium)\
            .send_keys_to_element(nomprenom, "age")\
            .perform()
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        divs = self.selenium.find_elements(By.CSS_SELECTOR, "#ui-id-1 div")
        self.assertEqual(
            {d.text for d in divs},
            {
                'Vagennius Manël',
                'Trodéxès de Collagène Nacer',
                'Agecanonix Suzie',
                'Trodéxès de Collagène Gael',
            }
        )
        # un va cliquer sur 'Trodéxès de Collagène Gael'
        tcg = [d for d in divs if d.text =='Trodéxès de Collagène Gael'][0]
        tcg.click()
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        # vérification du nombre de livres prêtés
        lignes = self.selenium.find_elements(
            By.CSS_SELECTOR, "#tableau_prets tr")
        self.assertEqual(len(lignes), 11)
        # on va prêter le livre de math '12341' à Trodéxès de Collagène Gael
        # celui-ci est en surnombre
        l_input = self.selenium.find_element(By.CSS_SELECTOR, "#codelivre")
        l_input.send_keys("12341")
        l_input.send_keys(Keys.RETURN)
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        # on va cliquer "non" (on doit entendre un son d'avertissement)
        self.selenium.find_elements(By.CSS_SELECTOR, ".ui-button")[0].click()
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        # on prête le livre pour de bon
        self.selenium.find_elements(
            By.CSS_SELECTOR, ".actionbuttons")[0].click()
        #un peu plus d'attente : un message apparaît puis disparaît
        ActionChains(self.selenium).pause(3).perform()
        # vérification du nombre de livres prêtés
        # contrairement à Firefox, selenium+chrome ne conserve pas
        # le contenu du champ "nomprenom" quand la page est rafraîchie
        self.selenium.find_element(By.ID, "nomprenom")\
                     .send_keys("Trodéxès de Collagène Gael\n")
        #un peu d'attente
        ActionChains(self.selenium).pause(.5).perform()
        lignes = self.selenium.find_elements(
            By.CSS_SELECTOR, "#tableau_prets tr")
        self.assertEqual(len(lignes), 12)

        # vérifions le nombre d'élèves dans la même classe
        meme_classe_lignes = self.selenium.find_elements(
            By.CSS_SELECTOR, ".liste-scroll-y tr")
        self.assertEqual(len(meme_classe_lignes), 30)
        # vérifions si la ligne de Trodéxès de Collagène Gael est
        # sélectionnée en gris
        td_eleve = self.selenium.find_elements(
            By.CSS_SELECTOR, ".liste-scroll-y tr td.meme-eleve")[0]
        self.assertEqual(td_eleve.text, "Trodéxès de Collagène Gael")

        if self.ui_visible:
            #un peu d'attente avant de tout effacer
            ActionChains(self.selenium).pause(6).perform()
        
